﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;

using DanielVaughan.Services;

using Microsoft.Phone.Shell;

namespace DanielVaughan.WindowsPhone7Unleashed.TodoList
{
	public class TodoItemViewModel : ViewModelBase
	{
		const string todoListUrl = "/TodoList/TodoListView.xaml";
		readonly ITodoService todoService;

		public TodoItemViewModel(ITodoService todoService)
		{
			this.todoService = ArgumentValidator.AssertNotNull(todoService, "todoService");

			saveCommand = new DelegateCommand<bool>(SaveItem);
			loadItemCommand = new DelegateCommand<int>(LoadItem);
			deleteCommand = new DelegateCommand(obj => DeleteItem());

			RegisterStatefulProperty(ApplicationStateType.Transient, () => TodoDescription);
			RegisterStatefulProperty(ApplicationStateType.Transient, () => TodoDueDate);
		}

		#region Loading

		TodoItem todoItem;

		void LoadItem(int itemId)
		{
			try
			{
				todoItem = todoService.GetTodoItem(itemId);
			}
			catch (KeyNotFoundException)
			{
				MessageService.ShowError("Item not found.");
			}

			TodoDescription = todoItem.Description;
			TodoDueDate = todoItem.DueDate;

			VisualState = "Update";
		}

		readonly DelegateCommand<int> loadItemCommand;

		public DelegateCommand<int> LoadItemCommand
		{
			get
			{
				return loadItemCommand;
			}
		}
		#endregion

		#region Saving

		void SaveItem(bool createTile)
		{
			if (string.IsNullOrWhiteSpace(todoDescription))
			{
				MessageService.ShowError("Please enter a description.", 
											"Required Field Missing");
				return;
			}

			if (todoItem == null)
			{
				todoItem = new TodoItem();
			}

			todoItem.Description = todoDescription;
			todoItem.DueDate = todoDueDate;

			todoService.AddOrUpdateItem(todoItem);

			StandardTileData tileData = TodoTileDataCreator.CreateTile(
											todoItem.Description, todoItem.DueDate);

			string url = string.Format("/TodoList/TodoItemView.xaml?{0}={1}",
										   TaskScheduler.TodoItemIdQueryKey,
										   todoItem.Id);

			if (createTile)
			{
				/* Creating a shell tile takes the user to the start experience. */
				ShellTile.Create(new Uri(url, UriKind.Relative), tileData);
			}
			else
			{
				ShellTile shellTile = ShellTile.ActiveTiles.Where(
					tile => tile.NavigationUri.ToString().Contains(url)).FirstOrDefault();

				if (shellTile != null)
				{
					shellTile.Update(tileData);
				}

				Navigate(todoListUrl);
			}
		}

		readonly DelegateCommand<bool> saveCommand;

		public DelegateCommand<bool> SaveCommand
		{
			get
			{
				return saveCommand;
			}
		}
		#endregion

		#region Deletion

		void DeleteItem()
		{
			if (todoItem == null)
			{
				throw new InvalidOperationException("Not in edit mode.");
			}

			todoService.RemoveTodoItem(todoItem);

			string tileQueryString = string.Format("{0}={1}",
			                                       TaskScheduler.TodoItemIdQueryKey,
			                                       todoItem.Id);

			ShellTile tile = ShellTile.ActiveTiles.FirstOrDefault(
				x => x.NavigationUri.ToString().Contains(tileQueryString));
			if (tile != null)
			{
				tile.Delete();
			}

			Navigate(todoListUrl);
		}

		readonly DelegateCommand deleteCommand;

		public ICommand DeleteCommand
		{
			get
			{
				return deleteCommand;
			}
		}
		#endregion

		string todoDescription;

		public string TodoDescription
		{
			get
			{
				return todoDescription;
			}
			set
			{
				Assign(() => TodoDescription, ref todoDescription, value);
			}
		}

		DateTime todoDueDate = DateTime.Now.AddDays(1);

		public DateTime TodoDueDate
		{
			get
			{
				return todoDueDate;
			}
			set
			{
				Assign(() => TodoDueDate, ref todoDueDate, value);
			}
		}

		string visualState = "Create";

		public string VisualState
		{
			get
			{
				return visualState;
			}
			private set
			{
				Assign(() => VisualState, ref visualState, value);
			}
		}
	}
}
