﻿using System;
using System.Collections.Generic;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Threading;
using System.Windows.Input;

using Microsoft.Phone.BackgroundTransfer;
using Microsoft.Phone.Scheduler;
using Microsoft.Phone.Shell;

namespace DanielVaughan.WindowsPhone7Unleashed.TodoList
{
	public class TodoListViewModel : ViewModelBase
	{
		string uploadUrl = "http://localhost:60182/BackupService/UploadFile/";
		string downloadUrl = "http://localhost:60182/Backups/";

		readonly IDeviceProperties deviceProperties;
		const string agentName = "UnleashedAgent";
		const string todoItemViewUrl = "/TodoList/TodoItemView.xaml";
		readonly ITodoService todoService;

		/* Uncomment the following line to test the mutex in the task agent. */
		//readonly Mutex mutex = new Mutex(true, "todo");

		public TodoListViewModel(
			ITodoService todoService, IDeviceProperties deviceProperties)
		{
			this.todoService = ArgumentValidator.AssertNotNull(todoService, "todoService");
			this.deviceProperties = ArgumentValidator.AssertNotNull(deviceProperties, "deviceProperties");

			editItemCommand = new DelegateCommand<int>(
				todoItemId => Navigate(todoItemViewUrl + "?TodoItemId=" + todoItemId));

			testAgentCommand = new DelegateCommand(
									obj => ScheduledActionService.LaunchForTest(
										agentName, TimeSpan.Zero));

			backupDatabaseCommand = new DelegateCommand(obj => BackupDatabase());
			restoreDatabaseCommand = new DelegateCommand(obj => RestoreDatabase());
			
			Load();

			UseMutexToConsumeCommonResource();
		}

		public void Load()
		{
			ThreadPool.QueueUserWorkItem(delegate { PopulateItems(); });

			PeriodicTask periodicTask = new PeriodicTask(agentName)
			{
				Description = "Updates todo item tiles.",
				//BeginTime = DateTime.Now.AddSeconds(20); not supported in current version
				ExpirationTime = DateTime.Now.AddDays(14)
			};
			
			if (ScheduledActionService.Find(agentName) != null)
			{
				ScheduledActionService.Remove(agentName);
			}

			/* This can only be called when the application 
			 * is running in the foreground. */
			ScheduledActionService.Add(periodicTask);
		}
		
		void PopulateItems()
		{
			try
			{
				IEnumerable<TodoItem> items = todoService.GetTodoItems();

				IEnumerable<Grouping<TodoItem>> groups 
					= from todoItem in items
				             orderby todoItem.DueDate
				             group todoItem by todoItem.DueDate.Date
				             into grouping
				             select new Grouping<TodoItem>(
											new DateGroupingKey(grouping.Key), 
											grouping.AsEnumerable());

				GroupedTodoItems = groups;
			}
			catch (Exception ex)
			{
				Message = "Unable to retrieve items.";
				Console.WriteLine(ex);
			}
		}

		IEnumerable<Grouping<TodoItem>> groupedTodoItems;

		public IEnumerable<Grouping<TodoItem>> GroupedTodoItems
		{
			get
			{
				return groupedTodoItems;
			}
			private set
			{
				Assign(() => GroupedTodoItems, ref groupedTodoItems, value);
			}
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			private set
			{
				Assign(() => Message, ref message, value);
			}
		}

		readonly DelegateCommand<int> editItemCommand;

		public DelegateCommand<int> EditItemCommand
		{
			get
			{
				return editItemCommand;
			}
		}

		readonly DelegateCommand testAgentCommand;

		public ICommand TestAgentCommand
		{
			get
			{
				return testAgentCommand;
			}
		}

		readonly DelegateCommand backupDatabaseCommand;

		public ICommand BackupDatabaseCommand
		{
			get
			{
				return backupDatabaseCommand;
			}
		}

		double progress;

		public double Progress
		{
			get
			{
				return progress;
			}
			private set
			{
				Assign(() => Progress, ref progress, value);
			}
		}

		bool progressVisible;

		public bool ProgressVisible
		{
			get
			{
				return progressVisible;
			}
			private set
			{
				Assign(() => ProgressVisible, ref progressVisible, value);
			}
		}

		const string localDatabaseName = "Todo.sdf";
		const string transferDirectory = "/shared/transfers";

		string GetUserId()
		{
			string id = deviceProperties.WindowsLiveAnonymousId;
			if (string.IsNullOrWhiteSpace(id))
			{
				id = "Emulator";
			}
			return id;
		}

		void BackupDatabase()
		{
			string uploadPath = transferDirectory + "/" + localDatabaseName;

			using (IsolatedStorageFile isolatedStorageFile
						= IsolatedStorageFile.GetUserStoreForApplication())
			{
				if (!isolatedStorageFile.FileExists(localDatabaseName))
				{
					throw new InvalidOperationException(
						"Database file does not exist in isolated storage.");
				}

				if (!isolatedStorageFile.DirectoryExists(transferDirectory))
				{
					isolatedStorageFile.CreateDirectory(transferDirectory);
				}

				isolatedStorageFile.CopyFile(localDatabaseName, uploadPath, true);
			}

			string deviceId = GetUserId();

			string remoteUrl = string.Format("{0}{1}/{2}",
			                                 uploadUrl,
			                                 deviceId,
			                                 localDatabaseName);

			Uri remoteUri = new Uri(remoteUrl, UriKind.Absolute);

			BackgroundTransferRequest request 
				= new BackgroundTransferRequest(remoteUri)
			            {
			                TransferPreferences = TransferPreferences.AllowBattery,
			                Method = "POST",
							UploadLocation = new Uri(uploadPath, UriKind.Relative)
			            };

			request.TransferStatusChanged += HandleUploadTransferStatusChanged;
			request.TransferProgressChanged += HandleTransferProgressChanged;
			//			BackgroundTransferRequest request2 = BackgroundTransferService.Find("request id string");
			//			if (request2 != null)
			//			{
			//				request.TransferStatusChanged += HandleUploadTransferStatusChanged;
			//				request.TransferProgressChanged += HandleTransferProgressChanged;
			//			}

			BackgroundTransferService.Add(request);
			Message = "Backing up data to cloud.";
			ProgressVisible = true;
		}

		void HandleUploadTransferStatusChanged(object sender, BackgroundTransferEventArgs e)
		{
			if (e.Request.TransferStatus == TransferStatus.Completed)
			{
				BackgroundTransferService.Remove(e.Request);

				ProgressVisible = false;

				if (e.Request.TransferError != null)
				{
					MessageService.ShowError("An error occured during backup.");
				}
				else
				{
					MessageService.ShowMessage("Backup successful.");
				}
			}
		}

		readonly DelegateCommand restoreDatabaseCommand;

		public ICommand RestoreDatabaseCommand
		{
			get
			{
				return restoreDatabaseCommand;
			}
		}
		
		void RestoreDatabase()
		{
			const string downloadPath = transferDirectory + "/" + localDatabaseName;

			using (IsolatedStorageFile isolatedStorageFile
						= IsolatedStorageFile.GetUserStoreForApplication())
			{
				if (!isolatedStorageFile.DirectoryExists(transferDirectory))
				{
					isolatedStorageFile.CreateDirectory(transferDirectory);
				}
			}

			string deviceId = GetUserId();

			string remoteUrl = string.Format("{0}{1}_{2}",
											 downloadUrl,
											 deviceId,
											 localDatabaseName);
			Uri remoteUri = new Uri(remoteUrl, UriKind.Absolute);
			Uri localUri = new Uri(downloadPath, UriKind.Relative);

			BackgroundTransferRequest request
				= new BackgroundTransferRequest(remoteUri, localUri)
				{
					TransferPreferences = TransferPreferences.AllowBattery,
					//Method = "Get", Get is the default value.
				};

			request.TransferStatusChanged += HandleDownloadTransferStatusChanged;
			request.TransferProgressChanged += HandleTransferProgressChanged;

			BackgroundTransferService.Add(request);
			Message = "Restoring data from cloud.";
			Progress = 0;
			ProgressVisible = true;
		}
		
		void HandleTransferProgressChanged(object sender, BackgroundTransferEventArgs e)
		{
			if (e.Request.BytesSent > 0)
			{
				Progress = (double)e.Request.TotalBytesToSend / e.Request.BytesSent;
			}
			else
			{
				Progress = 0;
			}
		}

		void HandleDownloadTransferStatusChanged(object sender, BackgroundTransferEventArgs e)
		{
			if (e.Request.TransferStatus == TransferStatus.Completed)
			{
				BackgroundTransferService.Remove(e.Request);

				ProgressVisible = false;

				if (e.Request.TransferError != null)
				{
					MessageService.ShowError("An error occured during restore.");
					return;
				}

				try
				{
					using (IsolatedStorageFile isolatedStorageFile
						= IsolatedStorageFile.GetUserStoreForApplication())
					{
						string downloadedFile = e.Request.DownloadLocation.OriginalString;
						isolatedStorageFile.CopyFile(downloadedFile, localDatabaseName, true);
					}

					todoService.Initialize();

					ClearPinnedItems();

					PopulateItems();
				}
				catch (Exception ex)
				{
					MessageService.ShowError("An error occured during restore.");
					return;
				}
				
				MessageService.ShowMessage("Restore successful.");
			}
		}

		void ClearPinnedItems()
		{
			string tileUrlSubstring = TaskScheduler.TodoItemIdQueryKey + "=";
			IEnumerable<ShellTile> tiles = ShellTile.ActiveTiles.Where(
					tile => tile.NavigationUri.ToString().Contains(tileUrlSubstring));
			foreach (var shellTile in tiles)
			{
				shellTile.Delete();
			}
		}

		void UseMutexToConsumeCommonResource()
		{
			CommonResourceExample resource = new CommonResourceExample();
			for (int i = 0; i < 100; i++)
			{
				ThreadPool.QueueUserWorkItem(
					delegate
						{
							string expectedContent = Guid.NewGuid().ToString();
							string actualContent = resource.WriteAndReadBackFile(expectedContent);
							if (actualContent != expectedContent)
							{
								System.Diagnostics.Debugger.Break();
							}
						});
			}
		}
	}
}
