﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading;
using System.Windows.Input;

using DanielVaughan.Data;

using Microsoft.Phone.BackgroundAudio;

namespace DanielVaughan.WindowsPhone7Unleashed.BackgroundAudio
{
	public class MainPageViewModel : ViewModelBase
	{
		readonly IBackgroundAudioPlayer player;
		Timer timer;

		public MainPageViewModel(
			IBackgroundAudioPlayer backgroundAudioPlayer, 
			IIsolatedStorageUtility isolatedStorageUtility)
		{
			player = ArgumentValidator.AssertNotNull(
						backgroundAudioPlayer, "backgroundAudioPlayer");
			ArgumentValidator.AssertNotNull(
						isolatedStorageUtility, "isolatedStorageUtility");

			player.PlayStateChanged += HandlePlayStateChanged;
			BackgroundAudioPlayerAgent.CopyAudioToIsolatedStorage(isolatedStorageUtility);

			playCommand = new DelegateCommand(obj => player.Play());
			pauseCommand = new DelegateCommand(obj => player.Pause());
			stopCommand = new DelegateCommand(obj => player.Stop());
			previousTrackCommand = new DelegateCommand(obj => player.SkipPrevious());
			nextTrackCommand = new DelegateCommand(obj => player.SkipNext());
			
			Refresh(false);

			timer = new Timer(HandleTimerTick, null, 3000, 1000);
		}

		void HandleTimerTick(object state)
		{
			if (player.PlayerState == PlayState.Playing)
			{
				OnPropertyChanged(() => Position);
			}
		}

		void HandlePlayStateChanged(object sender, EventArgs e)
		{
			Refresh();
		}

		void Refresh(bool setVisualState = true)
		{
			switch (player.PlayerState)
			{
				case PlayState.Playing:
					CanPause = true;
					TrackArtist = player.Track.Artist;
					TrackTitle = player.Track.Title;
					break;
				case PlayState.Paused:
					CanPause = false;
					break;
			}

			BufferingProgress = player.BufferingProgress;

			if (setVisualState)
			{
				VisualState = player.PlayerState.ToString("G");
			}

			if (player.Error != null)
			{
				MessageService.ShowError("A problem occured:" + player.Error);
			}
		}

		string trackTitle;

		public string TrackTitle
		{
			get
			{
				return trackTitle;
			}
			set
			{
				Assign(() => TrackTitle, ref trackTitle, value);
			}
		}

		string trackArtist;

		public string TrackArtist
		{
			get
			{
				return trackArtist;
			}
			set
			{
				Assign(() => TrackArtist, ref trackArtist, value);
			}
		}

		readonly DelegateCommand playCommand;

		public ICommand PlayCommand
		{
			get
			{
				return playCommand;
			}
		}

		readonly DelegateCommand pauseCommand;

		public ICommand PauseCommand
		{
			get
			{
				return pauseCommand;
			}
		}

		readonly DelegateCommand stopCommand;

		public ICommand StopCommand
		{
			get
			{
				return stopCommand;
			}
		}

		readonly DelegateCommand previousTrackCommand;

		public ICommand PreviousTrackCommand
		{
			get
			{
				return previousTrackCommand;
			}
		}

		readonly DelegateCommand nextTrackCommand;

		public ICommand NextTrackCommand
		{
			get
			{
				return nextTrackCommand;
			}
		}

		public double Volume
		{
			get
			{
				return player.Volume;
			}
			set
			{
				player.Volume = value;
				OnPropertyChanged(() => Volume);
			}
		}

		public double Position
		{
			get
			{
				if (player.Track == null || player.Track.Duration.TotalSeconds < 1)
				{
					return 0;
				}
				double result = player.Position.TotalSeconds 
									/ player.Track.Duration.TotalSeconds;
				return result;
			}
			set
			{
				if (player.Track != null)
				{
					double newSeconds = player.Track.Duration.TotalSeconds * value;
					TimeSpan newPosition = TimeSpan.FromSeconds(newSeconds);
					player.Position = newPosition;
				}
				OnPropertyChanged(() => Position);
			}
		}

		bool canPause;

		public bool CanPause
		{
			get
			{
				return canPause;
			}
			private set
			{
				Assign(() => CanPause, ref canPause, value);
			}
		}

		double bufferingProgress;

		public double BufferingProgress
		{
			get
			{
				return bufferingProgress;
			}
			private set
			{
				Assign(() => BufferingProgress, ref bufferingProgress, value);
			}
		}

		string visualState = PlayState.Stopped.ToString("G");

		public string VisualState
		{
			get
			{
				return visualState;
			}
			private set
			{
				Assign(() => VisualState, ref visualState, value);
			}
		}
	}
}
