﻿using System;
using System.Diagnostics;
using System.Threading;
using System.Windows;
using System.Windows.Media.Imaging;

using Microsoft.Xna.Framework;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public static class BitmapUtility
	{
		/// <summary>
		/// Finds the edges of an image using the Sobel operator.
		/// </summary>
		/// <param name="bitmapSource">The bitmap source.</param>
		/// <param name="resultAction">The action to invoke 
		/// once the task is complete.</param>
		/// <returns></returns>
		public static void FindEdgesUsingSobel(BitmapSource bitmapSource, 
			Action<ResultEventArgs<WriteableBitmap>> resultAction)
		{
			ArgumentValidator.AssertNotNull(bitmapSource, "bitmapSource");
			ArgumentValidator.AssertNotNull(resultAction, "resultAction");

			DateTime startTime = DateTime.Now;

			WriteableBitmap sourceBitmap = new WriteableBitmap(bitmapSource);
			int[] sourcePixels = sourceBitmap.Pixels;
			int imageWidth = bitmapSource.PixelWidth;
			int imageHeight = bitmapSource.PixelHeight;

			ThreadPool.QueueUserWorkItem(
				delegate
				{
					try
					{
						int[] resultPixels = FindEdgesUsingSobelCore(
												sourcePixels,
												imageWidth,
												imageHeight);
						Deployment.Current.Dispatcher.BeginInvoke(
							delegate
							{
								WriteableBitmap resultBitmap 
									= new WriteableBitmap(bitmapSource);

								for (int i = 0; i < resultPixels.Length; i++)
								{
									resultBitmap.Pixels[i] = resultPixels[i];
								}
								var timeToComplete = DateTime.Now - startTime;
								Debug.WriteLine("Conversion took: " + timeToComplete);

								resultAction(new ResultEventArgs<WriteableBitmap>(resultBitmap));
							});
						
					}
					catch (Exception ex)
					{
						resultAction(new ResultEventArgs<WriteableBitmap>(null, ex));
					}          		
			    });
		}

		static int[] FindEdgesUsingSobelCore(int[] sourcePixels, int imageWidth, int imageHeight)
		{
			/* Sobel Matrices */
			int[,] gx = new[,] { { -1, 0, 1 }, { -2, 0, 2 }, { -1, 0, 1 } };
			int[,] gy = new[,] { { 1, 2, 1 }, { 0, 0, 0 }, { -1, -2, -1 } };

			int[] resultPixels = new int[sourcePixels.Length];

			for (int i = 1; i < imageHeight - 1; i++)
			{
				for (int j = 1; j < imageWidth - 1; j++)
				{
					float newX = 0;
					float newY = 0;
					float c;
					for (int hw = -1; hw < 2; hw++)
					{
						for (int wi = -1; wi < 2; wi++)
						{
							int row = j + wi;
							int column = i + hw;
							int position = column * imageWidth + row;
							int pixel = sourcePixels[position];
							var bytes = BitConverter.GetBytes(pixel);
							c = (bytes[0] + bytes[1] + bytes[2]) / 3;
							int index0 = hw + 1;
							int index1 = wi + 1;
							newX += gx[index0, index1] * c;
							newY += gy[index0, index1] * c;
						}
					}

					int resultPosition = i * imageWidth + j;
					int colorIntValue;

					if (newX * newX + newY * newY > 16384 /* 128 * 128 */)
					{
						colorIntValue = ToInt(Color.White);
					}
					else
					{
						colorIntValue = ToInt(Color.Black);
					}
					resultPixels[resultPosition] = colorIntValue;
				}
			}

			return resultPixels;
		}

		static int ToInt(Color color)
		{
			return (color.A << 24) | (color.R << 16) 
						| (color.G << 8) | (color.B << 0);
		}
	}

	public class ResultEventArgs<T> : EventArgs
	{
		public T Result { get; private set; }
		public Exception Error { get; private set; }

		public ResultEventArgs(T result, Exception error = null)
		{
			Result = result;
			Error = error;
		}
	}
}
