﻿using System;
using System.ComponentModel;
using System.IO;
using System.Threading;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;

using DanielVaughan.Utilities;

using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using Microsoft.Xna.Framework.Media;

namespace DanielVaughan.WindowsPhone7Unleashed.ExtrasExample
{
	public partial class MainPage : PhoneApplicationPage
	{
		string imageName = "WindowsPhone7UnleashedTestImage";
		readonly PhotoChooserTask photoChooserTask = new PhotoChooserTask();
        volatile bool canSave;

		public MainPage()
		{
			InitializeComponent();

            saveCommand = new DelegateCommand(obj => SaveImage(), obj => canSave);

			DataContext = this;

			photoChooserTask.Completed += HandlePhotoChooserTaskCompleted;
		}

		void HandlePhotoChooserTaskCompleted(object sender, PhotoResult e)
		{
			if (e.TaskResult != TaskResult.OK)
			{
				return;
			}
			imageName = e.OriginalFileName;

		    if (!string.IsNullOrWhiteSpace(imageName))
		    {
		        int index = imageName.LastIndexOf(@"\");
		        if (index >= 0)
		        {
		            imageName = "Unleashed_" + imageName.Substring(index + 1);
		        }
		    }
		    else
		    {
		        imageName = string.Format(
                    "Unleashed_{0:yyyy-MM-dd-HH-mm-ss}.jpg", DateTime.Now);
		    }
		    Stream imageStream = e.ChosenPhoto;
			ProcessImage(imageStream);
		}

		protected override void OnNavigatedTo(NavigationEventArgs e)
		{
			base.OnNavigatedTo(e);

			/* Equivalent to: Environment.DeviceType == DeviceType.Emulator */
			if (EnvironmentValues.UsingEmulator)
			{
				ProcessTestImage();
			}
			else
			{
				string token;
				if (NavigationContext.QueryString.TryGetValue("token", out token))
				{
					ProcessImageUsingToken(token);
				}
				else
				{
                    if (image.Source == null)
                    {
                        photoChooserTask.Show();
                    }
				}
			}
		}
	
		void ProcessImageUsingToken(string token)
		{
			imageName = token;

			Picture picture;
			using (MediaLibrary mediaLibrary = new MediaLibrary())
			{
				picture = mediaLibrary.GetPictureFromToken(token);
			}
			Stream imageStream = picture.GetImage();
			ProcessImage(imageStream);
		}

		void ProcessImage(Stream imageStream)
		{
            BitmapImage sourceImage = new BitmapImage();
            sourceImage.CreateOptions = BitmapCreateOptions.None;
			sourceImage.SetSource(imageStream);
			image.Source = sourceImage;

		    canSave = false;
            saveCommand.RaiseCanExecuteChanged();

			progressBar.Visibility = Visibility.Visible;
			BitmapUtility.FindEdgesUsingSobel(sourceImage, HandleEdgesFound);
		}

		void ProcessTestImage()
		{
			const string testImageUrl = @"/DanielVaughan.WindowsPhone7Unleashed" 
				+ ".ExtrasExample;component/Images/TestImage.jpg";
		
			var imageUri = new Uri(testImageUrl, UriKind.Relative);

			using (Stream stream = Application.GetResourceStream(imageUri).Stream)
			{
				BitmapSource bitmapSource = new BitmapImage();
				bitmapSource.SetSource(stream);
				image.Source = bitmapSource;

				progressBar.Visibility = Visibility.Visible;
				BitmapUtility.FindEdgesUsingSobel(bitmapSource, HandleEdgesFound);
			}
		}

		void HandleEdgesFound(ResultEventArgs<WriteableBitmap> obj)
		{
			progressBar.Visibility = Visibility.Collapsed;

			if (obj.Error != null)
			{
				MessageBox.Show("Unable to find edges.");
				return;
			}
			image.Source = obj.Result;

            canSave = true;
            saveCommand.RaiseCanExecuteChanged();
		}

		readonly DelegateCommand saveCommand;

		public ICommand SaveCommand
		{
			get
			{
				return saveCommand;
			}
		}

		void SaveImage()
		{
			var writeableBitmap = (WriteableBitmap)image.Source;
			if (writeableBitmap == null)
			{
				return;
			}

			const int jpegQuality = 90;

			using (MemoryStream memoryStream = new MemoryStream())
			{
				writeableBitmap.SaveJpeg(
					memoryStream,
					writeableBitmap.PixelWidth, // target width
					writeableBitmap.PixelHeight, // target height
					0, // orientation. 
					jpegQuality);

				memoryStream.Seek(0, SeekOrigin.Begin);
				using (MediaLibrary mediaLibrary = new MediaLibrary())
				{
					Picture savedPicture = mediaLibrary.SavePicture(imageName, memoryStream);
				}
			}

			MessageBox.Show("Image saved to Saved Pictures.");
		}

        //protected override void OnBackKeyPress(CancelEventArgs e)
        //{
        //    ScreenCaptureUtility.CaptureImageToMediaLibrary(this);
        //    e.Cancel = true;
        //    base.OnBackKeyPress(e);
        //}
	}
}