﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2009, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
	This file is part of Calcium..

	Calcium is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Calcium is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with DanielVaughan's base library.  If not, see http://www.gnu.org/licenses/.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2008-12-28 19:52:25Z</CreationDate>
    <Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace DanielVaughan.Collections
{
	public static class Extensions
	{
		/// <summary>
		/// Examines each item in the double collection 
		/// and returns the index of the greatest value.
		/// </summary>
		/// <param name="values">The values.</param>
		/// <returns></returns>
		public static int GetIndexOfGreatest(this IEnumerable<double> values)
		{
			ArgumentValidator.AssertNotNull(values, "values");

			int result = -1;
			int count = 0;
			double highest = -1;
			foreach (var d in values)
			{
				if (d > highest)
				{
					highest = d;
					result = count;
				}
				count++;
			}
			return result;
		}

		/// <summary>
		/// Removes all items from the list that do not pass the filter condition.
		/// </summary>
		/// <typeparam name="T">The generic type of the list.</typeparam>
		/// <param name="list">The list to remove from.</param>
		/// <param name="filter">The filter to evaluate each item with.</param>
		/// <returns>The removed items.</returns>
		public static IEnumerable<T> RemoveAllAndReturnItems<T>(this IList<T> list, Func<T, bool> filter)
		{
			for (int i = 0; i < list.Count; i++)
			{
				if (filter(list[i]))
				{
					var item = list[i];
					list.Remove(list[i]);
					yield return item;
				}
			}
		}

		/// <summary>
		/// Adds the items from one set of items to the specified list.
		/// </summary>
		/// <typeparam name="T">The item type.</typeparam>
		/// <param name="toList">The destination list.</param>
		/// <param name="fromList">The source list.</param>
		public static void AddRange<T>(this IList<T> toList, IEnumerable<T> fromList)
		{
			ArgumentValidator.AssertNotNull(toList, "toList");
			ArgumentValidator.AssertNotNull(fromList, "fromList");

			foreach (var item in fromList)
			{
				toList.Add(item);
			}
		}

		/// <summary>
		/// Adds the items from one set of items to the specified list.
		/// </summary>
		/// <typeparam name="T">The item type.</typeparam>
		/// <param name="toList">The destination list.</param>
		/// <param name="fromList">The source list.</param>
		public static void AddRange(this IList toList, IEnumerable fromList)
		{
			ArgumentValidator.AssertNotNull(toList, "toList");
			ArgumentValidator.AssertNotNull(fromList, "fromList");

			foreach (var item in fromList)
			{
				toList.Add(item);
			}
		}

		/// <summary>
		/// Determines whether the collection is null or contains no elements.
		/// </summary>
		/// <typeparam name="T">The IEnumerable type.</typeparam>
		/// <param name="enumerable">The enumerable, which may be null or empty.</param>
		/// <returns>
		/// 	<c>true</c> if the IEnumerable is null or empty; otherwise, <c>false</c>.
		/// </returns>
		public static bool IsNullOrEmpty<T>(this IEnumerable<T> enumerable)
		{
			if (enumerable == null)
			{
				return true;
			}
			/* If this is a list, use the Count property. 
			 * The Count property is O(1) while IEnumerable.Count() is O(N). */
			var collection = enumerable as ICollection<T>;
			if (collection != null)
			{
				return collection.Count < 1;
			}
			return enumerable.Any();
		}

		/// <summary>
		/// Determines whether the collection is null or contains no elements.
		/// </summary>
		/// <typeparam name="T">The IEnumerable type.</typeparam>
		/// <param name="collection">The collection, which may be null or empty.</param>
		/// <returns>
		/// 	<c>true</c> if the IEnumerable is null or empty; otherwise, <c>false</c>.
		/// </returns>
		public static bool IsNullOrEmpty<T>(this ICollection<T> collection)
		{
			if (collection == null)
			{
				return true;
			}
			return collection.Count < 1;
		}
	}
}
