﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2009, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
	This file is part of Calcium.

	Calcium is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Calcium is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Calcium.  If not, see http://www.gnu.org/licenses/.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2010-10-21 15:34:42Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class CommandBehavior
	{
		ICommand command;
		object commandParameter;
		readonly WeakReference targetReference;
		readonly EventHandler canExecuteChangedHandler;

		public CommandBehavior(FrameworkElement targetElement)
		{
			targetReference = new WeakReference(targetElement);
			canExecuteChangedHandler = (o, args) => UpdateEnabledState();
		}

		public void ExecuteCommand(object sender, EventArgs args)
		{
			ExecuteCommand();
		}

		protected virtual void ExecuteCommand()
		{
			if (command != null)
			{
				command.Execute(CommandParameter);
			}
		}

		public ICommand Command
		{
			get
			{
				return command;
			}
			set
			{
				if (command == value)
				{
					return;
				}

				if (command != null)
				{
					command.CanExecuteChanged -= canExecuteChangedHandler;
				}

				command = value;
				if (command != null)
				{
					command.CanExecuteChanged += canExecuteChangedHandler;
					UpdateEnabledState();
				}
			}
		}

		public object CommandParameter
		{
			get
			{
				return commandParameter;
			}
			set
			{
				if (commandParameter != value)
				{
					commandParameter = value;
					UpdateEnabledState();
				}
			}
		}

		string eventName;

		public string EventName
		{
			get
			{
				return eventName;
			}
			set
			{
				if (eventName != value)
				{
					eventName = value;
					if (string.IsNullOrEmpty(eventName))
					{
						return;
					}
					FrameworkElement element = TargetElement;
					if (element == null)
					{
						return;
					}
					EventInfo eventInfo = element.GetType().GetEvent(eventName);
					if (eventInfo == null)
					{
						throw new ArgumentException(string.Format(
							"Event name '{0}' not found on FrameworkElement '{1}'", 
							eventName, element));
					}

					if (System.ComponentModel.DesignerProperties.IsInDesignTool)
					{
						return;
					}

					Delegate handler = Delegate.CreateDelegate(
						eventInfo.EventHandlerType, this, "ExecuteCommand");
					eventInfo.AddEventHandler(element, handler);
				}
			}
		}

		protected FrameworkElement TargetElement
		{
			get
			{
				return (FrameworkElement)targetReference.Target;
			}
		}

		protected virtual void UpdateEnabledState()
		{
			if (TargetElement == null)
			{
				command = null;
				commandParameter = null;
			}
			else if (command != null)
			{
				var targetControl = TargetElement as Control;
				if (targetControl != null)
				{
					targetControl.IsEnabled = command.CanExecute(commandParameter);
				}
			}
		}
	}
}