﻿using System;
using System.Collections.Generic;
using System.IO;
using System.IO.IsolatedStorage;
using System.Runtime.Serialization;
using System.Windows.Media.Imaging;

namespace DanielVaughan.WindowsPhone7Unleashed.Data.ImageCache
{
	/* Based on work by Peter Nowak (http://blogs.compactframework.de/Peter.Nowak/2010/08/12/ldquoIntelligentrdquo+Image+Caching+For+WP+7.aspx)  
	 * and Ingo Rama (http://weblogs.thinktecture.com/ingo/). */
	static class ImageCache
	{
		static string imageCacheFile = "ImageCache.data";

		public static string ImageCacheFile
		{
			get
			{
				return imageCacheFile;
			}
			set
			{
				imageCacheFile = value;
			}
		}

		static readonly object imageCacheLock = new object();
		static Dictionary<string, ImageCacheItem> imageCacheDictionary;

		internal static void AddCacheItem(string url, ImageCacheItem imageCacheItem)
		{
			lock (imageCacheLock)
			{
				imageCacheDictionary.Add(url, imageCacheItem);
			}
		}

		public static BitmapImage GetImage(BitmapImage image)
		{
			ArgumentValidator.AssertNotNull(image, "image");

			string url = image.UriSource.ToString();

			if (Environment.OSVersion.Platform == PlatformID.Win32NT)
			{
				image.UriSource = new Uri(url);
				return image;
			}

			if (imageCacheDictionary == null)
			{
				lock (imageCacheLock)
				{
					if (imageCacheDictionary == null)
					{
						LoadCachedImageInfo();
					}
				}
			}

			ImageCacheItem cacheItem;
			if (imageCacheDictionary.TryGetValue(url, out cacheItem))
			{
				if (DateTime.Compare(DateTime.Now, cacheItem.Expiration) >= 0)
				{
					/* Item has expired. */
					ImageDownloader.DownloadImage(url, image, cacheItem);
				}
				else
				{
					string imageFileName = cacheItem.LocalFilename;
					IsolatedStorageFile storageFile 
						= IsolatedStorageFile.GetUserStoreForApplication();
					IsolatedStorageFileStream imageFile = storageFile.OpenFile(
						imageFileName, FileMode.Open);

					image.SetSource(imageFile);
				}
			}
			else
			{
				var item = new ImageCacheItem();
				ImageDownloader.DownloadImage(url, image, item);
			}

			return image;
		}

		static void LoadCachedImageInfo()
		{
			IsolatedStorageFile storageFile = 
				IsolatedStorageFile.GetUserStoreForApplication();

			if (!storageFile.FileExists(imageCacheFile))
			{
				imageCacheDictionary = new Dictionary<string, ImageCacheItem>();
				return;
			}

			bool success = false;

			using (IsolatedStorageFileStream fileStream = storageFile.OpenFile(
				imageCacheFile, FileMode.Open))
			{
				DataContractSerializer serializer = new DataContractSerializer(
					typeof(Dictionary<string, ImageCacheItem>));

				try
				{
					imageCacheDictionary 
						= (Dictionary<string, ImageCacheItem>)serializer
															.ReadObject(fileStream);
					success = true;
				}
				catch (Exception ex)
				{
					Console.WriteLine("Unable to read image cache file. " + ex);
				}
			}

			if (!success)
			{
				/* An error occured, so delete the existing cached item. */
				storageFile.DeleteFile(imageCacheFile);
				imageCacheDictionary = new Dictionary<string, ImageCacheItem>();
			}
		}

		internal static void Save()
		{
			IsolatedStorageFile storageFile = 
				IsolatedStorageFile.GetUserStoreForApplication();

			using (IsolatedStorageFileStream fileStream 
				= storageFile.CreateFile(imageCacheFile))
			{
				DataContractSerializer serializer = new DataContractSerializer(
					typeof(Dictionary<string, ImageCacheItem>));
				serializer.WriteObject(fileStream, imageCacheDictionary);
				fileStream.Flush();
			}
		}
	}
}
