﻿using System;
using System.Diagnostics;
using System.IO;
using System.IO.IsolatedStorage;
using System.Net;
using System.Security.Cryptography;
using System.Text;
using System.Windows;
using System.Windows.Media.Imaging;

namespace DanielVaughan.WindowsPhone7Unleashed.Data.ImageCache
{
	/* Based on work by Peter Nowak (http://blogs.compactframework.de/Peter.Nowak/2010/08/12/ldquoIntelligentrdquo+Image+Caching+For+WP+7.aspx)  
	 * and Ingo Rama (http://weblogs.thinktecture.com/ingo/). */
	static class ImageDownloader
	{
		static double expirationDays = 1.0;

		public static double ExpirationDays
		{
			get
			{
				return expirationDays;
			}
			set
			{
				expirationDays = value;
			}
		}

		public static void DownloadImage(string imageUrl, BitmapImage image,
			ImageCacheItem imageCacheItem)
		{
			ArgumentValidator.AssertNotNull(imageCacheItem, "item");
			string filename = CreateUniqueFilename(imageUrl);
			imageCacheItem.LocalFilename = filename;

			var asyncDataTransfer = new AsyncDataTransfer
									{
										ImageCacheItem = imageCacheItem,
										Image = image
									};

			var httpWebRequest = (HttpWebRequest)WebRequest.Create(imageUrl);
			if (imageCacheItem.ImageId != null)
			{
				httpWebRequest.Headers["If-None-Match"] = imageCacheItem.ImageId;
			}
			asyncDataTransfer.Request = httpWebRequest;

			httpWebRequest.BeginGetResponse(RequestCallback, asyncDataTransfer);
		}

		static void RequestCallback(IAsyncResult asyncResult)
		{
			if (!asyncResult.IsCompleted)
			{
				return;
			}

			AsyncDataTransfer transfer = (AsyncDataTransfer)asyncResult.AsyncState;
			try
			{
				HttpWebResponse response = 
					(HttpWebResponse)transfer.Request.EndGetResponse(asyncResult);

				IsolatedStorageFile storageFile = 
					IsolatedStorageFile.GetUserStoreForApplication();

				if (response.StatusCode == HttpStatusCode.NotModified)
				{
					Deployment.Current.Dispatcher.BeginInvoke(
						() =>
						{
							string fileName = transfer.ImageCacheItem.LocalFilename;
							transfer.Image.SetSource(
								storageFile.OpenFile(fileName, FileMode.Open));
						});
					return;
				}

				transfer.ImageCacheItem.ImageId = response.Headers["ETag"];

				transfer.ImageCacheItem.Expiration 
					= response.Headers["Expires"] != null
					? DateTime.Parse(response.Headers["Expires"])
					: DateTime.Now.AddDays(expirationDays);

				Stream responseStream = response.GetResponseStream();
				using (var writer = new BinaryWriter(
					storageFile.CreateFile(transfer.ImageCacheItem.LocalFilename)))
				{
					var b = new byte[4096];
					int read;
					while ((read = responseStream.Read(b, 0, b.Length)) > 0)
					{
						writer.Write(b, 0, read);
					}
					writer.Flush();
				}

				Deployment.Current.Dispatcher.BeginInvoke(
					() =>
						{
							string fileName = transfer.ImageCacheItem.LocalFilename;
							transfer.Image.SetSource(
								storageFile.OpenFile(fileName, FileMode.Open));
						});

				ImageCache.AddCacheItem(
					transfer.Request.RequestUri.ToString(), transfer.ImageCacheItem);

				ImageCache.Save();
			}
			catch (Exception ex)
			{
				Debug.WriteLine("Unable to cache image. " + ex);
			}
		}

		static string CreateUniqueFilename(string url)
		{
			string extension = Path.GetExtension(url);
			byte[] textToHash = Encoding.UTF8.GetBytes(url);
			var sha1Managed = new SHA1Managed();
			byte[] hash = sha1Managed.ComputeHash(textToHash);
			return BitConverter.ToString(hash) + extension;
		}
	}
}
