﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2010, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions are met:
			* Redistributions of source code must retain the above copyright
			  notice, this list of conditions and the following disclaimer.
			* Redistributions in binary form must reproduce the above copyright
			  notice, this list of conditions and the following disclaimer in the
			  documentation and/or other materials provided with the distribution.
			* Neither the name of the <organization> nor the
			  names of its contributors may be used to endorse or promote products
			  derived from this software without specific prior written permission.

		THIS SOFTWARE IS PROVIDED BY Daniel Vaughan ''AS IS'' AND ANY
		EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL Daniel Vaughan BE LIABLE FOR ANY
		DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
		(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
		LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
		ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2011-06-06 10:10:43Z</CreationDate>
</File>
*/
#endregion

using System;
using System.Collections.Generic;
using System.IO;
using System.IO.IsolatedStorage;
using System.Windows;

namespace DanielVaughan.Data
{
	public interface IIsolatedStorageUtility
	{
		void CopyApplicationResourceToIsolatedStorage(
			string inResourceName, string outFilename);

		void CopyApplicationResourcesToIsolatedStorage(
			IEnumerable<KeyValuePair<string, string>> sourceToDestinationList);

		Stream GetApplicationResourceStream(Uri resourceUri);

		bool FileExists(string path);
	}

	public interface ISettingsService
	{
		TSetting GetSetting<TSetting>(string key, TSetting defaultValue);
		void SetSetting<T>(string key, T value);
	}

	public class IsolatedStorageUtility : IIsolatedStorageUtility, ISettingsService
	{
		public void CopyApplicationResourceToIsolatedStorage(
							string inResourceName, string outFilename)
		{
			ArgumentValidator.AssertNotNull(inResourceName, "inResourceName");
			ArgumentValidator.AssertNotNullOrWhiteSpace(outFilename, "outFilename");

			Uri uri = new Uri(inResourceName, UriKind.Relative);

			string destination = outFilename;
			int seperatorIndex = destination.LastIndexOf("/");
			if (seperatorIndex == destination.Length - 1)
			{
				throw new InvalidOperationException(
					string.Format("Destination '{0}' should not end with '/'",
									destination));
			}

			string directory = null;
			if (seperatorIndex != -1)
			{
				directory = destination.Substring(0, seperatorIndex);
			}

			using (Stream resourceStream = Application.GetResourceStream(uri).Stream)
			{
				using (IsolatedStorageFile isolatedStorageFile
							= IsolatedStorageFile.GetUserStoreForApplication())
				{
					if (!string.IsNullOrWhiteSpace(directory)
						&& !isolatedStorageFile.DirectoryExists(directory))
					{
						isolatedStorageFile.CreateDirectory(directory);
					}

					using (IsolatedStorageFileStream outStream
						= isolatedStorageFile.OpenFile(destination, FileMode.Create))
					{
						resourceStream.CopyTo(outStream);
					}
				}
			}
		}

		public void CopyApplicationResourcesToIsolatedStorage(
							IEnumerable<KeyValuePair<string, string>> sourceToDestinationList)
		{
			ArgumentValidator.AssertNotNull(sourceToDestinationList, "sourceToDestinationList");

			using (IsolatedStorageFile isolatedStorageFile
						= IsolatedStorageFile.GetUserStoreForApplication())
			{
				foreach (var sourceAndDestination in sourceToDestinationList)
				{
					Uri uri = new Uri(sourceAndDestination.Key, UriKind.Relative);
					using (Stream resourceStream = Application.GetResourceStream(uri).Stream)
					{
						string destination = sourceAndDestination.Value;
						if (string.IsNullOrWhiteSpace(destination))
						{
							throw new ArgumentException(
								string.Format("Key '{0}' has null pair Value. A destination must be specified."));
						}

						int seperatorIndex = destination.LastIndexOf("/");
						if (seperatorIndex == destination.Length - 1)
						{
							throw new InvalidOperationException(
								string.Format("Destination '{0}' should not end with '/'",
												destination));
						}
						string directory = null;
						if (seperatorIndex != -1)
						{
							directory = destination.Substring(0, seperatorIndex);
						}

						if (!string.IsNullOrWhiteSpace(directory) 
							&& !isolatedStorageFile.DirectoryExists(directory))
						{
							isolatedStorageFile.CreateDirectory(directory);
						}

						using (IsolatedStorageFileStream outStream
							= isolatedStorageFile.OpenFile(destination, FileMode.Create))
						{
							resourceStream.CopyTo(outStream);
						}
					}
				}
			}
		}

		public Stream GetApplicationResourceStream(Uri resourceUri)
		{
			Stream resourceStream = Application.GetResourceStream(resourceUri).Stream;
			return resourceStream;
		}

		public bool FileExists(string path)
		{
			ArgumentValidator.AssertNotNull(path, "path");

			using (IsolatedStorageFile isolatedStorageFile 
				= IsolatedStorageFile.GetUserStoreForApplication())
			{
				return isolatedStorageFile.FileExists(path);
			}
		}

		public TSetting GetSetting<TSetting>(string key, TSetting defaultValue)
		{
			TSetting result = defaultValue;
			if (IsolatedStorageSettings.ApplicationSettings.Contains(key))
			{
				object settingValue = IsolatedStorageSettings.ApplicationSettings[key];
				result = (TSetting)settingValue;
			}
			return result;
		}

		public void SetSetting<T>(string key, T value)
		{
			IsolatedStorageSettings.ApplicationSettings[key] = value;
			IsolatedStorageSettings.ApplicationSettings.Save();
		}
	}
}
