﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2010, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions are met:
			* Redistributions of source code must retain the above copyright
			  notice, this list of conditions and the following disclaimer.
			* Redistributions in binary form must reproduce the above copyright
			  notice, this list of conditions and the following disclaimer in the
			  documentation and/or other materials provided with the distribution.
			* Neither the name of the <organization> nor the
			  names of its contributors may be used to endorse or promote products
			  derived from this software without specific prior written permission.

		THIS SOFTWARE IS PROVIDED BY <copyright holder> ''AS IS'' AND ANY
		EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
		DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
		(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
		LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
		ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2011-06-03 12:09:17Z</CreationDate>
</File>
*/
#endregion

using System;
using System.Data.Linq;
using System.IO;
using System.IO.IsolatedStorage;
using System.Windows;

namespace DanielVaughan.Data
{
	public class DatabaseUtility
	{
		string databaseFileName;
		readonly LocalDatabaseMode localDatabaseMode;

		public DatabaseUtility(string databaseFileName, 
			LocalDatabaseMode localDatabaseMode = LocalDatabaseMode.ReadWrite)
		{
			this.localDatabaseMode = localDatabaseMode;
			this.databaseFileName = ArgumentValidator.AssertNotNullOrWhiteSpace(
												databaseFileName, "databaseFileName");
		}

		public void InitializeDatabase(DataContext dataContext, bool wipe = false)
		{
			ArgumentValidator.AssertNotNull(dataContext, "dataContext");

			if (localDatabaseMode != LocalDatabaseMode.ReadWrite 
				&& localDatabaseMode != LocalDatabaseMode.Exclusive)
			{
				return;
				//throw new InvalidOperationException(
				//"Database is not writable. Its mode is " + localDatabaseMode);
			}

			if (wipe && dataContext.DatabaseExists())
			{
				dataContext.DeleteDatabase();
			}

			if (!dataContext.DatabaseExists())
			{
				dataContext.CreateDatabase();
			}
		}

		public void DeleteDatabase()
		{
			using (IsolatedStorageFile isolatedStorageFile
						= IsolatedStorageFile.GetUserStoreForApplication())
			{
				DeleteDatabase(isolatedStorageFile);
			}
		}

		void DeleteDatabase(IsolatedStorageFile isolatedStorageFile)
		{
			string name = GetFileNameWithoutPrefix(databaseFileName);

			if (isolatedStorageFile.FileExists(name))
			{
				isolatedStorageFile.DeleteFile(name);
			}
		}

		static string GetFileNameWithoutPrefix(string name)
		{
			if (name.StartsWith("isostore:", StringComparison.OrdinalIgnoreCase))
			{
				return name.Substring(9);
			}
			return name;
		}

		public bool DatabaseExists()
		{
			string name = GetFileNameWithoutPrefix(databaseFileName);

			using (IsolatedStorageFile isolatedStorageFile 
						= IsolatedStorageFile.GetUserStoreForApplication())
			{
				return isolatedStorageFile.FileExists(name);
			}
		}

		public string DataSource
		{
			get
			{
				return databaseFileName;
			}
		}

		public string DatabaseFileName
		{
			get
			{
				return databaseFileName;
			}
			set
			{
				databaseFileName = value;
			}
		}

		public string DatabasePassword { get; set; }

		public string ConnectionString
		{
			get
			{
				if (string.IsNullOrWhiteSpace(DatabasePassword))
				{
					return string.Format("Data Source='{0}';Mode={1}", 
						DataSource, localDatabaseMode.ToConnectionStringValue());
				}
				return string.Format("Data Source='{0}';Password='{1}';Mode={2}",
										DataSource, 
										DatabasePassword, 
										localDatabaseMode.ToConnectionStringValue());
			}
		}
	}
}