﻿using System;
using System.Device.Location;

using DanielVaughan.ComponentModel;

using Microsoft.Phone.Reactive;

namespace DanielVaughan.WindowsPhone7Unleashed.GeoLocation
{
	public class GeoPositionSampler<T> : NotifyPropertyChangeBase, IDisposable
	{
		readonly IDisposable subscription;
		readonly IGeoPositionWatcher<T> watcher;

		public GeoPositionSampler(
			IGeoPositionWatcher<T> watcher, int sampleIntervalMs)
		{
			this.watcher = ArgumentValidator.AssertNotNull(watcher, "watcher");
            ArgumentValidator.AssertGreaterThanOrEqualTo(0, sampleIntervalMs, "sampleIntervalMs");

			IObservable<IEvent<GeoPositionChangedEventArgs<T>>> observable
				= Observable.FromEvent<GeoPositionChangedEventArgs<T>>(
					ev => watcher.PositionChanged += ev,
					ev => watcher.PositionChanged -= ev);

			IObservable<IEvent<GeoPositionChangedEventArgs<T>>> sampled 
				= observable.Sample(
					TimeSpan.FromMilliseconds(sampleIntervalMs));

			subscription = sampled.Subscribe(
				args =>
					{
						GeoPosition<T> position = args.EventArgs.Position;
						Location = position.Location;
						TimeStamp = position.Timestamp;
					});

			watcher.StatusChanged += WatcherOnStatusChanged;
		}

		void WatcherOnStatusChanged(
			object o, GeoPositionStatusChangedEventArgs args)
		{
			Status = args.Status;
		}

		T location;

		public T Location
		{
			get
			{
				return location;
			}
			private set
			{
				Assign("Location", ref location, value);
			}
		}

		DateTimeOffset timeStamp;

		public DateTimeOffset TimeStamp
		{
			get
			{
				return timeStamp;
			}
			private set
			{
				Assign("TimeStamp", ref timeStamp, value);
			}
		}

		GeoPositionStatus status;

		public GeoPositionStatus Status
		{
			get
			{
				return status;
			}
			set
			{
				Assign("Status", ref status, value);
			}
		}

		#region IDisposable Implementation

		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		bool disposed;

		void Dispose(bool disposing)
		{
			/* Check to see if Dispose has already been called. */
			if (disposed)
			{
				return;
			}

			/* If disposing equals true, dispose all managed 
			 * and unmanaged resources. */
			if (disposing)
			{
				if (subscription != null)
				{
					try
					{
						subscription.Dispose();
					}
					catch (Exception)
					{
						/* Suppress. */
					}

					try
					{
						if (watcher != null)
						{
							watcher.StatusChanged -= WatcherOnStatusChanged;
						}
					}
					catch (Exception)
					{
						/* Suppress. */
					}
				}
			}

			/* Disposing has been done. */
			disposed = true;
		}

		/// <summary>
		/// Releases unmanaged resources and performs other cleanup operations 
		/// before the <see cref="GeoPositionSampler{T}"/> 
		/// is reclaimed by garbage collection.
		/// Use C# destructor syntax for finalization code. 
		/// This destructor will run only if the Dispose method does not get called.
		/// It gives your base class the opportunity to finalize. 
		/// Do not provide destructors in types derived from this class.
		/// </summary>
		~GeoPositionSampler()
		{
			/* Do not re-create Dispose clean-up code here. 
			 * Calling Dispose(false) is optimal 
			 * in terms of readability and maintainability.*/
			Dispose(false);
		}

		#endregion
	}
}
