﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Device.Location;
using System.Threading;
using System.Linq;

namespace DanielVaughan.WindowsPhone7Unleashed.GeoLocation
{
	public class MockGeoCoordinateWatcher : IGeoPositionWatcher<GeoCoordinate>, 
		INotifyPropertyChanged
	{
		bool started;
		readonly object startedLock = new object();
		Thread workerThread;
		bool firstRun = true;

		public MockGeoCoordinateWatcher() : this(500)
		{
			/* Intentionally left blank. */
		}

		public MockGeoCoordinateWatcher(int defaultDelay)
		{
			InitializationMs = defaultDelay;
			FixDelayMs = defaultDelay;
			PositionChangeDelayMs = defaultDelay;
		}

		public MockGeoCoordinateWatcher(GeoPositionAccuracy accuracy) : this()
		{
			Accuracy = accuracy;
		}

		public GeoPositionAccuracy	Accuracy { get; set; }
		public int					PositionChangeDelayMs { get; set; }
		public int					FixDelayMs { get; set; }
		public int					InitializationMs { get; set; }

		public event PropertyChangedEventHandler PropertyChanged;

		protected void OnPropertyChanged(string propertyName)
		{
			PropertyChanged.Raise(
				this, new PropertyChangedEventArgs(propertyName));
		}

		public void Start()
		{
			if (LockedOperations.TrySetTrue(ref started, startedLock))
			{
				workerThread = new Thread(WalkPath);
				workerThread.Start();
			}
		}

		public void Start(bool suppressPermissionPrompt)
		{
			Start();
		}

		public bool TryStart(bool suppressPermissionPrompt, TimeSpan timeout)
		{
			try
			{
				Start(suppressPermissionPrompt);
			}
			catch (Exception)
			{
				return false;
			}
			return true;
		}

		public GeoPositionPermission Permission { get; private set; }

		GeoPosition<GeoCoordinate> position;

		public GeoPosition<GeoCoordinate> Position
		{
			get
			{
				return position;
			}
			private set
			{
				position = value;
				OnPositionChanged(
					new GeoPositionChangedEventArgs<GeoCoordinate>(value));
				OnPropertyChanged("Position");
			}
		}

		GeoPositionStatus status = GeoPositionStatus.Initializing;

		public GeoPositionStatus Status
		{
			get
			{
				return status;
			}
			private set
			{
				if (status == value)
				{
					return;
				}
				status = value;
				OnStatusChanged(new GeoPositionStatusChangedEventArgs(value));
				OnPropertyChanged("Status");
			}
		}

		public event EventHandler<
			GeoPositionChangedEventArgs<GeoCoordinate>> PositionChanged;

		protected void OnPositionChanged(
			GeoPositionChangedEventArgs<GeoCoordinate> args)
		{
			PositionChanged.Raise(this, args);
		}

		public event EventHandler<GeoPositionStatusChangedEventArgs> StatusChanged;

		protected void OnStatusChanged(GeoPositionStatusChangedEventArgs args)
		{
			StatusChanged.Raise(this, args);
		}

		void WalkPath()
		{
			if (wayPoints.Count < 1)
			{
				List<GeoCoordinate> defaultWayPoints = GetDefaultCoordinates();
				wayPoints.AddRange(defaultWayPoints);
			}

			if (firstRun)
			{
				Status = GeoPositionStatus.Initializing;
				Wait(InitializationMs);
			}

			Status = GeoPositionStatus.NoData;
			Wait(FixDelayMs);

            GeoCoordinate coordinate = wayPoints.First();

			if (firstRun)
			{
				Position = new GeoPosition<GeoCoordinate>(
									DateTimeOffset.Now, coordinate);
				firstRun = false;
			}
			Status = GeoPositionStatus.Ready;
            Wait(PositionChangeDelayMs);
			int index = 1;

			while (started)
			{
				if (wayPoints != null && wayPoints.Count > index)
				{
					coordinate = wayPoints[index++];
				}
				SetPosition(coordinate);
				Wait(PositionChangeDelayMs);
			}
		}

		static readonly object waitLock = new object();

		static void Wait(int delayMs)
		{
			if (delayMs > 0)
			{
				lock (waitLock)
				{
					Monitor.Wait(waitLock, delayMs);
				}
			}
		}

		readonly List<GeoCoordinate> wayPoints = new List<GeoCoordinate>();

		public List<GeoCoordinate> WayPoints
		{
			get
			{
				return wayPoints;
			}
		}

		void SetPosition(GeoCoordinate coordinate)
		{
			Position = new GeoPosition<GeoCoordinate>(DateTimeOffset.Now, coordinate);
		}

		public void Stop()
		{
			if (LockedOperations.TrySetFalse(ref started, startedLock))
			{
				workerThread = null;
			}			
		}

		static List<GeoCoordinate> GetDefaultCoordinates()
		{
			return new List<GeoCoordinate>
			        {
						new GeoCoordinate(46.209477, 6.143074), 
						new GeoCoordinate(46.213754, 6.145778), 
						new GeoCoordinate(46.219129, 6.148996), 
						new GeoCoordinate(46.224651, 6.148138), 
						new GeoCoordinate(46.231034, 6.149426), 
						new GeoCoordinate(46.237149, 6.148996), 
						new GeoCoordinate(46.243175, 6.151657), 
						new GeoCoordinate(46.249318, 6.150756), 
						new GeoCoordinate(46.254986, 6.155047), 
						new GeoCoordinate(46.256974, 6.156421), 
						new GeoCoordinate(46.260891, 6.159167), 
						new GeoCoordinate(46.2646, 6.161957), 
						new GeoCoordinate(46.267863, 6.164017), 
						new GeoCoordinate(46.2708, 6.165862), 
						new GeoCoordinate(46.274182, 6.167493), 
						new GeoCoordinate(46.276555, 6.169682), 
						new GeoCoordinate(46.282784, 6.165905), 
						new GeoCoordinate(46.288418, 6.165776), 
						new GeoCoordinate(46.290346, 6.165776), 
						new GeoCoordinate(46.294853, 6.166892), 
						new GeoCoordinate(46.297611, 6.168909), 
						new GeoCoordinate(46.300665, 6.174531), 
						new GeoCoordinate(46.30443, 6.179895), 
						new GeoCoordinate(46.307869, 6.183157), 
						new GeoCoordinate(46.313709, 6.188779), 
						new GeoCoordinate(46.316703, 6.193714), 
						new GeoCoordinate(46.319845, 6.195602), 
						new GeoCoordinate(46.321949, 6.196761), 
						new GeoCoordinate(46.329269, 6.201096), 
						new GeoCoordinate(46.337269, 6.202855)
			        };
		}
	}
}
