﻿// Copyright (C) Microsoft Corporation. All Rights Reserved.
// This code released under the terms of the Microsoft Public License
// (Ms-PL, http://opensource.org/licenses/ms-pl.html).

/* Original code by David Anson http://blogs.msdn.com/b/delay/ */

using System;
using System.Diagnostics.CodeAnalysis;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Animation;

using Microsoft.Phone.Controls;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	/// <summary>
	/// PhoneApplicationFrame subclass that animates device orientation changes.
	/// </summary>
	public class AnimateOrientationChangesFrame : PhoneApplicationFrame
	{
		/// <summary>
		/// Stores the Transform used to do the rotation.
		/// </summary>
		readonly RotateTransform rotateTransform = new RotateTransform();

		/// <summary>
		/// Stores the Storyboard used to change the Progress value.
		/// </summary>
		readonly Storyboard progressStoryboard = new Storyboard();

		/// <summary>
		/// Stores the Animation used to change the Progress value.
		/// </summary>
		readonly DoubleAnimation progressAnimation = new DoubleAnimation();

		/// <summary>
		/// Stores the "from" state.
		/// </summary>
		readonly OrientationState from = new OrientationState();

		/// <summary>
		/// Stores the "to" state.
		/// </summary>
		readonly OrientationState to = new OrientationState();

		/// <summary>
		/// Stores a reference to the "ClientArea" template part.
		/// </summary>
		UIElement clientArea;

		/// <summary>
		/// Stores the last computed size from OnProgressChanged.
		/// </summary>
		Size lastSize;

		/// <summary>
		/// Initializes a new instance of the AnimateOrientationChangesFrame class.
		/// </summary>
		public AnimateOrientationChangesFrame()
		{
			// Set up transform (note: completely replaces any existing transform(s))
			RenderTransform = rotateTransform;
			RenderTransformOrigin = new Point(0.5, 0.5);

			// Set up animation
			progressAnimation.From = 0;
			progressAnimation.To = 1;
			Storyboard.SetTarget(progressAnimation, this);
			Storyboard.SetTargetProperty(progressAnimation, new PropertyPath("Progress"));
			progressStoryboard.Children.Add(progressAnimation);

			// Initialize variables
			EasingFunction = new QuarticEase(); // Initialized here to avoid a single shared instance

			// Hook events
			SizeChanged += HandleSizeChanged;
			OrientationChanged += HandleOrientationChanged;
		}
		
		/// <summary>
		/// Gets or sets a value indicating whether animation is enabled.
		/// </summary>
		public bool AnimationEnabled
		{
			get
			{
				return (bool)GetValue(AnimationEnabledProperty);
			}
			set
			{
				SetValue(AnimationEnabledProperty, value);
			}
		}

		/// <summary>
		/// Identifies the IsAnimationEnabled DependencyProperty.
		/// </summary>
		public static readonly DependencyProperty AnimationEnabledProperty 
			= DependencyProperty.Register("AnimationEnabled", typeof(bool),
		            typeof(AnimateOrientationChangesFrame),
		            new PropertyMetadata(true));

		/// <summary>
		/// Gets or sets a value indicating the duration of the orientation change animation.
		/// </summary>
		public TimeSpan Duration
		{
			get
			{
				return (TimeSpan)GetValue(DurationProperty);
			}
			set
			{
				SetValue(DurationProperty, value);
			}
		}

		/// <summary>
		/// Identifies the Duration DependencyProperty.
		/// </summary>
		public static readonly DependencyProperty DurationProperty 
			= DependencyProperty.Register("Duration", typeof(TimeSpan), 
						typeof(AnimateOrientationChangesFrame),
		               new PropertyMetadata(TimeSpan.FromSeconds(0.4)));

		/// <summary>
		/// Gets or sets a value indicating the IEasingFunction to use for the orientation change animation.
		/// </summary>
		public IEasingFunction EasingFunction
		{
			get
			{
				return (IEasingFunction)GetValue(EasingFunctionProperty);
			}
			set
			{
				SetValue(EasingFunctionProperty, value);
			}
		}

		/// <summary>
		/// Identifies the EasingFunction DependencyProperty.
		/// </summary>
		public static readonly DependencyProperty EasingFunctionProperty 
			= DependencyProperty.Register("EasingFunction", typeof(IEasingFunction),
		            typeof(AnimateOrientationChangesFrame),
		            new PropertyMetadata(null));

		/// <summary>
		/// Identifies the Progress DependencyProperty.
		/// </summary>
		static readonly DependencyProperty ProgressProperty 
			= DependencyProperty.Register("Progress", typeof(double), 
					typeof(AnimateOrientationChangesFrame),
					new PropertyMetadata(0.0, OnProgressChanged));

		/// <summary>
		/// Handles changes to the Progress property.
		/// </summary>
		/// <param name="o">Event source.</param>
		/// <param name="e">Event arguments.</param>
		static void OnProgressChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
		{
			((AnimateOrientationChangesFrame)o).OnProgressChanged((double)e.NewValue);
		}

		/// <summary>
		/// Handles changes to the Progress property.
		/// </summary>
		/// <param name="newValue">New value.</param>
		void OnProgressChanged( /*double oldValue,*/ double newValue)
		{
			// Update rotation
			rotateTransform.Angle = from.Angle + (newValue * (to.Angle - from.Angle));
			// Invalidate only if size has changed
			double width = from.Width + (newValue * (to.Width - from.Width));
			double height = from.Height + (newValue * (to.Height - from.Height));
			var size = new Size(Math.Round(width), Math.Round(height));
			if (size != lastSize)
			{
				lastSize = size;
				InvalidateMeasure();
			}
		}

		/// <summary>
		/// Called when the element's Template changes.
		/// </summary>
		public override void OnApplyTemplate()
		{
			base.OnApplyTemplate();
			// Get the template part
			clientArea = base.GetTemplateChild("ClientArea") as UIElement;
		}

		/// <summary>
		/// Handles the SizeChanged event.
		/// </summary>
		/// <param name="sender">Event source.</param>
		/// <param name="e">Event arguments.</param>
		void HandleSizeChanged(object sender, SizeChangedEventArgs e)
		{
			// Capture new size
			from.Width = e.NewSize.Width;
			from.Height = e.NewSize.Height;
			lastSize = e.NewSize;

			// Measure/Arrange can be called before the SizeChanged event fires
			InvalidateMeasure();
		}

		/// <summary>
		/// Handles the OrientationChanged event.
		/// </summary>
		/// <param name="sender">Event source.</param>
		/// <param name="e">Event arguments.</param>
		[SuppressMessage("Microsoft.Naming", "CA2204:Literals should be spelled correctly", 
			MessageId = "PageOrientation", Justification = "Spelled correctly.")]
		void HandleOrientationChanged(object sender, OrientationChangedEventArgs e)
		{
			// Capture current/before angle and size
			from.Angle = rotateTransform.Angle;
			from.Width = lastSize.Width;
			from.Height = lastSize.Height;
			progressStoryboard.Stop();

			// Determine new angle
			switch (e.Orientation)
			{
				case PageOrientation.PortraitUp:
					to.Angle = 0;
					break;
				case PageOrientation.LandscapeLeft:
					to.Angle = 90;
					break;
				case PageOrientation.LandscapeRight:
					to.Angle = -90;
					break;
				case PageOrientation.PortraitDown:
					to.Angle = 180;
					break;
				default:
					throw new NotSupportedException("Unknown PageOrientation value.");
			}

			// Determine new size
			double actualWidth = ActualWidth;
			double actualHeight = ActualHeight;
			bool toPortrait = (0 == (to.Angle % 180));
			to.Width = toPortrait ? actualWidth : actualHeight;
			to.Height = toPortrait ? actualHeight : actualWidth;

			if (AnimationEnabled)
			{
				// Animate the rotation
				progressAnimation.Duration = Duration;
				progressAnimation.EasingFunction = EasingFunction;
				progressStoryboard.Begin();
			}
			else
			{
				// Snap to the rotation (with guaranteed property change)
				SetValue(ProgressProperty, 0.0);
				SetValue(ProgressProperty, 1.0);
			}
		}

		/// <summary>
		/// Handles measuring the children of this element.
		/// </summary>
		/// <param name="availableSize">Available size.</param>
		/// <returns>Desired size.</returns>
		protected override Size MeasureOverride(Size availableSize)
		{
			if (null != clientArea)
			{
				// Adjust measure size to transition size
				var newValue = (double)GetValue(ProgressProperty);
				double width = from.Width + (newValue * (to.Width - from.Width));
				double height = from.Height + (newValue * (to.Height - from.Height));
				clientArea.Measure(new Size(width, height));
			}
			// Return default size
			return availableSize;
		}

		/// <summary>
		/// Handles arranging the children of this element.
		/// </summary>
		/// <param name="finalSize">Size to arrange to.</param>
		/// <returns>Used size.</returns>
		protected override Size ArrangeOverride(Size finalSize)
		{
			if (null != clientArea)
			{
				// Adjust arrange size to transition size
				var newValue = (double)GetValue(ProgressProperty);
				double width = from.Width + (newValue * (to.Width - from.Width));
				double height = from.Height + (newValue * (to.Height - from.Height));
				clientArea.Arrange(new Rect((finalSize.Width - width) / 2, (finalSize.Height - height) / 2, width, height));
			}
			// Return default size
			return finalSize;
		}

		/// <summary>
		/// Stores state variables for orientation changes.
		/// </summary>
		class OrientationState
		{
			/// <summary>
			/// Gets or sets the Width.
			/// </summary>
			public double Width { get; set; }

			/// <summary>
			/// Gets or sets the Height.
			/// </summary>
			public double Height { get; set; }

			/// <summary>
			/// Gets or sets the Angle.
			/// </summary>
			public double Angle { get; set; }
		}
	}
}
