﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading;

using Microsoft.Phone.Notification;

namespace DanielVaughan.WindowsPhone7Unleashed.PushNotification
{
	public class PushNotificationSubscriber : IDisposable
	{
		Timer retryTimer;
		HttpNotificationChannel channel;
		readonly string channelName;
		readonly string serviceName;
		readonly Action<string> registerWithServer;

		int retryDueTimeMs = 30 * 1000; /* 30 seconds. */
		public int RetryTimeMs
		{
			get
			{
				return retryDueTimeMs;
			}
			set
			{
				ArgumentValidator.AssertGreaterThan(value, 0, "value");
				retryDueTimeMs = value;
			}
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="PushNotificationSubscriber"/> class.
		/// </summary>
		/// <param name="registerWithServer">The method used to register 
		/// a push notification URL with a third party server. 
		/// The third party server will normally be your server. Can be <c>null</c>.</param>
		public PushNotificationSubscriber(string channelName, string serviceName, 
			Action<string> registerWithServer)
		{
			this.channelName = channelName;
			this.serviceName = serviceName;
			this.registerWithServer = registerWithServer;
		}

		public void Subscribe()
		{
			try
			{
				channel = HttpNotificationChannel.Find(channelName);
				if (channel == null)
				{
					channel = string.IsNullOrEmpty(serviceName) 
						? new HttpNotificationChannel(channelName)
						: new HttpNotificationChannel(channelName, serviceName);
				}
				else
				{
					channel.UnbindToShellTile();
					channel.UnbindToShellToast();
					channel.Close();
					Subscribe();
					return;
				}

				UnsubscribeFromChannelEvents();				
				SubscribeToChannelEvents();
				
				try
				{
					channel.Open();
				}
				catch (Exception ex)
				{
					channel.UnbindToShellTile();
					channel.UnbindToShellToast();
					channel.Close();
					WaitAndRetryChannelConnect();
					return;
				}

				/* Toast Notifications. */
				if (!channel.IsShellToastBound)
				{
					channel.BindToShellToast();
				}

				/* Tile Notifications. */
				if (!channel.IsShellTileBound)
				{
					channel.BindToShellTile();
				}

                if (channel.ChannelUri != null)
				{
					RegisterChannel(channel.ChannelUri);
				}
			}
			catch (Exception ex)
			{
				Debug.WriteLine("Unable to subscribe. " + ex.ToString());
				WaitAndRetryChannelConnect();
			}
		}
		
		public void Unsubscribe()
		{
			UnBindToNotifications();
            UnsubscribeFromChannelEvents();
		}

		void UnBindToNotifications()
		{
			/* Toast Notifications. */
			channel.UnbindToShellToast();
			/* Tile Notifications. */
			channel.UnbindToShellTile();
		}

		void SubscribeToChannelEvents()
		{
			/* ChannelUriUpdated occurs when the Push Notification URI changes. 
			 * This can occur at anytime.*/
			channel.ChannelUriUpdated += channel_ChannelUriUpdated; 
			/* Raw notification received. */
			channel.HttpNotificationReceived += channel_HttpNotificationReceived;
			/* Toast notification received. */
			channel.ShellToastNotificationReceived += channel_ShellToastNotificationReceived; 
			/* Errors */
			channel.ErrorOccurred += HandleChannelErrorOccurred;
		}

		void UnsubscribeFromChannelEvents()
		{
			/* ChannelUriUpdated occurs when the Push Notification URL changes. 
			 * This can occur at anytime.*/
			channel.ChannelUriUpdated -= channel_ChannelUriUpdated;
			/* Raw notification received. */
			channel.HttpNotificationReceived -= channel_HttpNotificationReceived;
			/* Toast notification received. */
			channel.ShellToastNotificationReceived -= channel_ShellToastNotificationReceived;
			/* Errors */
			channel.ErrorOccurred -= HandleChannelErrorOccurred;
		}

		void RegisterChannel(Uri uri)
		{
			if (registerWithServer != null)
			{
				registerWithServer(uri.ToString());
			}
		}

		void WaitAndRetryChannelConnect()
		{
			if (retryTimer == null)
			{
				/* Create a timer and have it fire once. */
				retryTimer = new Timer(o => Subscribe(), null, retryDueTimeMs, 
					Timeout.Infinite);
			}
			else
			{
				/* The timer is changed to fire once after the period expires. */
				retryTimer.Change(retryDueTimeMs, Timeout.Infinite);
			}
		}

		#region Channel event handlers

		void channel_HttpNotificationReceived(object sender, HttpNotificationEventArgs e)
		{
			OnHttpNotificationReceived(e);
		}

		void channel_ShellToastNotificationReceived(object sender, NotificationEventArgs e)
		{
//			string text1 = e.Collection["wp:Text1"];
//			string text2 = e.Collection["wp:Text2"];
//			Debug.WriteLine("Toast notification received");
//			foreach (KeyValuePair<string, string> pair in e.Collection)
//			{
//				Debug.WriteLine(string.Format("Key: {0} Value: {1}", 
//					pair.Key, pair.Value));
//			}
			OnShellNotificationReceived(e);
		}

		void HandleChannelErrorOccurred(object sender, NotificationChannelErrorEventArgs e)
		{
			OnExceptionOccurred(e);

			/* The following switch statement can be used 
			 * to handle particular error types. */
			switch (e.ErrorType)
			{
				case ChannelErrorType.ChannelOpenFailed:
					// ...
					break;
				case ChannelErrorType.MessageBadContent:
					// ...
					break;
				case ChannelErrorType.NotificationRateTooHigh:
					// ...
					break;
				case ChannelErrorType.PayloadFormatError:
					// ...
					break;
				case ChannelErrorType.PowerLevelChanged:
					// ...
                    if (e.ErrorAdditionalData == (int)ChannelPowerLevel.LowPowerLevel 
                            || e.ErrorAdditionalData == (int)ChannelPowerLevel.CriticalLowPowerLevel)
                    {
                        /* Power level is too low. */
                    }
					break;
			}
		}

		void channel_ChannelUriUpdated(object sender, NotificationChannelUriEventArgs e)
		{
			RegisterChannel(e.ChannelUri);
		}

		#endregion

		#region event ChannelUriUpdated

		event EventHandler<NotificationChannelUriEventArgs> channelUriUpdated;

		public event EventHandler<NotificationChannelUriEventArgs> ChannelUriUpdated
		{
			add
			{
				channelUriUpdated += value;
			}
			remove
			{
				channelUriUpdated -= value;
			}
		}

		protected void OnChannelUriUpdated(NotificationChannelUriEventArgs e)
		{
			var eventCopy = channelUriUpdated;
			if (eventCopy != null)
			{
				eventCopy(this, e);
			}
		}

		#endregion

		#region event HttpNotificationReceived

		event EventHandler<HttpNotificationEventArgs> httpNotificationReceived;

		public event EventHandler<HttpNotificationEventArgs> HttpNotificationReceived
		{
			add
			{
				httpNotificationReceived += value;
			}
			remove
			{
				httpNotificationReceived -= value;
			}
		}

		protected void OnHttpNotificationReceived(HttpNotificationEventArgs e)
		{
			var eventCopy = httpNotificationReceived;
			if (eventCopy != null)
			{
				eventCopy(this, e);
			}
		}

		#endregion

//		#region event ShellEntryPointNotificationReceived
//
//		event EventHandler<NotificationEventArgs> shellTileNotificationReceived;
//
//		public event EventHandler<NotificationEventArgs> ShellTileNotificationReceived
//		{
//			add
//			{
//				shellTileNotificationReceived += value;
//			}
//			remove
//			{
//				shellTileNotificationReceived -= value;
//			}
//		}
//
//		protected void OnShellTileNotificationReceived(NotificationEventArgs e)
//		{
//			var eventCopy = shellTileNotificationReceived;
//			if (eventCopy != null)
//			{
//				eventCopy(this, e);
//			}
//		}
//
//		#endregion

		#region event ShellNotificationReceived

		event EventHandler<NotificationEventArgs> shellNotificationReceived;

		public event EventHandler<NotificationEventArgs> ShellNotificationReceived
		{
			add
			{
				shellNotificationReceived += value;
			}
			remove
			{
				shellNotificationReceived -= value;
			}
		}

		protected void OnShellNotificationReceived(NotificationEventArgs e)
		{
			var eventCopy = shellNotificationReceived;
			if (eventCopy != null)
			{
				eventCopy(this, e);
			}
		}

		#endregion

		#region event ExceptionOccurred

		event EventHandler<NotificationChannelErrorEventArgs> exceptionOccurred;

		public event EventHandler<NotificationChannelErrorEventArgs> ErrorOccurred
		{
			add
			{
				exceptionOccurred += value;
			}
			remove
			{
				exceptionOccurred -= value;
			}
		}

		protected void OnExceptionOccurred(NotificationChannelErrorEventArgs e)
		{
			var eventCopy = exceptionOccurred;
			if (eventCopy != null)
			{
				eventCopy(this, e);
			}
		}

		#endregion

		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		bool disposed;

		void Dispose(bool disposing)
		{
			/* Check to see if Dispose has already been called. */
			if (disposed)
			{
				return;
			}

			/* If disposing equals true, dispose all managed 
			 * and unmanaged resources. */
			if (disposing)
			{
				if (retryTimer != null)
				{
					try
					{
						retryTimer.Dispose();
					}
					catch (Exception ex)
					{
						Debug.WriteLine(ex);
					}
				}

				if (channel != null)
				{
					try
					{
						channel.Dispose();
					}
					catch (Exception ex)
					{
						Debug.WriteLine(ex);
					}
				}
			}

			/* Disposing has been done. */
			disposed = true;
		}

		/// <summary>
		/// Releases unmanaged resources and performs other cleanup operations 
		/// before the <see cref="PushNotificationSubscriber"/> 
		/// is reclaimed by garbage collection.
		/// Use C# destructor syntax for finalization code. 
		/// This destructor will run only if the Dispose method does not get called.
		/// It gives your base class the opportunity to finalize. 
		/// Do not provide destructors in types derived from this class.
		/// </summary>
		~PushNotificationSubscriber()
		{
			/* Do not re-create Dispose clean-up code here. 
			 * Calling Dispose(false) is optimal 
			 * in terms of readability and maintainability.*/
			Dispose(false);
		}

	}
}