﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2009, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
	This file is part of Calcium.

	Calcium is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Calcium is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Calcium.  If not, see http://www.gnu.org/licenses/.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2009-04-25 14:43:57Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;

namespace DanielVaughan.Services
{
	/// <summary>
	/// The result from a user when prompted with a question 
	/// that has three options: Yes, No, and Cancel.
	/// </summary>
	public enum YesNoCancelQuestionResult
	{
		/// <summary>
		/// The user selected 'yes.'
		/// </summary>
		Yes,
		/// <summary>
		/// The user selected 'no.'
		/// </summary>
		No,
		/// <summary>
		/// The user selected 'cancel.'
		/// </summary>
		Cancel
	}

	/// <summary>
	/// The result from a user when prompted with a question 
	/// that has two options: Ok and Cancel.
	/// </summary>
	public enum OkCancelQuestionResult
	{
		/// <summary>
		/// The user selected 'Ok.'
		/// </summary>
		Ok,
		/// <summary>
		/// The user selected 'cancel.'
		/// </summary>
		Cancel
	}

	/// <summary>
	/// Is used to indicate the which messages are to shown 
	/// depending on the user's filter level.
	/// </summary>
	public enum MessageImportance
	{
		/// <summary>
		/// All messages always shown.
		/// </summary>
		Low = 0,
		/// <summary>
		/// Some messages shown.
		/// </summary>
		Medium = 2,
		/// <summary>
		/// Only essential message are shown.
		/// </summary>
		High = 4
	}

	/// <summary>
	/// This service allows feedback to be gathered from a user.
	/// In the implementation this will generally be done using a modal dialog box.
	/// </summary>
	public interface IMessageService
	{
		/// <summary>
		/// Shows a message to a user that must be dismissed before continuing.
		/// </summary>
		/// <param name="message">The message to display.</param>
		/// <param name="caption">The caption to display on the dialog.</param>
		/// <param name="importanceThreshold">The minimum filter level 
		/// that a user must have in order to receive the message.</param>
		void ShowMessage(
			string message,
			string caption = null,
			MessageImportance importanceThreshold = MessageImportance.High,
			string details = null);

		/// <summary>
		/// Shows a warning message to a user that must be dismissed before continuing.
		/// </summary>
		/// <param name="message">The message to display.</param>
		/// <param name="caption">The caption to display on the dialog.</param>
		void ShowWarning(string message, string caption = null, string details = null);

		/// <summary>
		/// Shows an error message to a user that must be dismissed before continuing.
		/// </summary>
		/// <param name="message">The message to display.</param>
		/// <param name="caption">The caption to display on the dialog.</param>
		void ShowError(string message, string caption = null, string details = null);

		/// <summary>
		/// Asks the user a question where the valid response is either 'yes' or 'no'.
		/// </summary>
		/// <param name="question">The question text.</param>
		/// <param name="caption">The caption to display on the dialog.</param>
		/// <returns><code>true</code> if the user selected 'yes'; 
		/// <code>false</code> otherwise.</returns>
		bool AskYesNoQuestion(string question, string caption = null, string details = null);

		/// <summary>
		/// Asks the user a question where the valid response 
		/// is either 'yes', 'no', or 'cancel.'
		/// </summary>
		/// <param name="question">The question text.</param>
		/// <param name="caption">The caption to display on the dialog.</param>
		/// <returns>The answer from the user. 
		/// <see cref="YesNoCancelQuestionResult"/></returns>
		YesNoCancelQuestionResult AskYesNoCancelQuestion(
			string question, string caption = null, string details = null);

		/// <summary>
		/// Asks the user a question where the valid response 
		/// is either 'ok' or 'cancel.'
		/// </summary>
		/// <param name="question">The question text.</param>
		/// <param name="caption">The caption to display on the dialog.</param>
		/// <returns><code>true</code> if the user selected 'ok'; 
		/// <code>false</code> otherwise.</returns>
		bool AskOkCancelQuestion(string question, string caption = null, string details = null);

		void AskQuestion<TResponse>(
				IQuestion<TResponse> question,
				Action<QuestionResult<TResponse>> resultHandler)
					where TResponse : class, IQuestionResult;
	}

	public class QuestionResult<TResponse> where TResponse : class, IQuestionResult
	{
		public IQuestion<TResponse> Question { get; private set; }
		public TResponse Response { get; private set; }
		public Exception Error { get; private set; }

		public QuestionResult(TResponse result, IQuestion<TResponse> question, Exception error = null)
		{
			Response = result;
			Question = ArgumentValidator.AssertNotNull(question, "question");
			Error = error;
		}

		public bool HasValidResponse
		{
			get
			{
				return Error == null && Response != null && Response.HasValidAnswer;
			}
		}
	}

	public interface IQuestion<TResponse>
	{
		/* Placeholder. */
	}

	public interface IQuestionResult
	{
		bool HasValidAnswer { get; }
	}
}
