﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2009, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
	This file is part of Calcium.

	Calcium is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Calcium is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Calcium.  If not, see http://www.gnu.org/licenses/.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2009-04-25 14:43:57Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;
using System.Threading;
using System.Windows;
using System.Windows.Threading;

namespace DanielVaughan.Services.Implementation
{
	public partial class MessageService : MessageServiceBase
	{
		public override MessageResult ShowCustomDialog(
			string message, 
			string caption,
			MessageButton messageButton, 
			MessageImage messageImage, 
			MessageImportance? importanceThreshold,
			string details)
		{
			/* If the importance threshold has been specified 
			 * and it's less than the minimum level required (the filter level) 
			 * then we don't show the message. */
			if (importanceThreshold.HasValue && importanceThreshold.Value < MinumumImportance)
			{
				return MessageResult.Ok;
			}

			/* This implementation has been simplified for Windows Phone Unleashed. */

			if (Deployment.Current.Dispatcher.CheckAccess())
			{	/* We are on the UI thread, and hence no need to invoke the call.*/
				var messageBoxResult = MessageBox.Show(message, caption,
					messageButton.TranslateToMessageBoxButton());
				return messageBoxResult.TranslateToMessageBoxResult();
			}

			MessageResult result = MessageResult.Ok;
			var context = new DispatcherSynchronizationContext(
				Deployment.Current.Dispatcher);
			context.Send(
				delegate
				{
					var messageBoxResult = MessageBox.Show(
												message, caption,
												messageButton.TranslateToMessageBoxButton());
					result = messageBoxResult.TranslateToMessageBoxResult();
				}, null);

			return result;
		}

		public override void AskQuestion<TResponse>(
			IQuestion<TResponse> question,
			Action<QuestionResult<TResponse>> resultHandler)
		{
			ArgumentValidator.AssertNotNull(question, "question");
			ArgumentValidator.AssertNotNull(resultHandler, "resultHandler");

			/* TODO: Add an extensibility point here so that new types of questions can be added using IoC. */

			var textResponseQuestion = question as TextResponseQuestion;
			var handler = resultHandler as Action<QuestionResult<TextResponse>>;

			if (textResponseQuestion == null || handler == null)
			{
				throw new NotImplementedException("Currently this method only supports TextResponse questions.");
			}

			AskQuestionWithTextResponse(textResponseQuestion, handler);
			//			if (!typeof(TextResponse).IsAssignableFrom(typeof(TResponse)))
			//			{
			//				throw new NotImplementedException("Currently this method only supports TextResponse questions.");
			//			}
		}

		void AskQuestionWithTextResponse(
			TextResponseQuestion textResponseQuestion,
			Action<QuestionResult<TextResponse>> resultHandler)
		{
				try
				{
					AskQuestionWithTextResponseCore(textResponseQuestion, resultHandler);
				}
				catch (Exception ex)
				{
					Console.WriteLine(ex);
					Deployment.Current.Dispatcher.BeginInvoke(
						delegate
						{
							resultHandler(new QuestionResult<TextResponse>(null, textResponseQuestion, ex));
						});
				}
		}

		void AskQuestionWithTextResponseCore(
			TextResponseQuestion textResponseQuestion,
			Action<QuestionResult<TextResponse>> resultHandler)
		{
			TextResponseWindow window = null;
			OkCancelQuestionResult questionResult;
			var textResponseViewModel
					= new TextResponseViewModel(textResponseQuestion);
			Deployment.Current.Dispatcher.BeginInvoke(
				delegate
				{
					try
					{
						window = new TextResponseWindow();
						window.Closed += delegate
						                 	{
						                 		bool dialogResult = window.DialogResult.HasValue && window.DialogResult.Value;
												questionResult = dialogResult ? OkCancelQuestionResult.Ok : OkCancelQuestionResult.Cancel;
												var textResponse = new TextResponse(questionResult, textResponseViewModel.Response);
												resultHandler(new QuestionResult<TextResponse>(textResponse, textResponseQuestion));
						                 	};
						window.DataContext = textResponseViewModel;
						window.Show();
					}
					catch (Exception ex)
					{
						Console.WriteLine(ex);
						resultHandler(new QuestionResult<TextResponse>(null, textResponseQuestion, ex));
					}
				});
		}
	}

	public class TextResponseViewModel
	{
		public TextResponseQuestion TextResponseQuestion { get; private set; }
		public string Response { get; set; }
		public string Question { get; private set; }

		public TextResponseViewModel(TextResponseQuestion textResponseQuestion)
		{
			TextResponseQuestion = textResponseQuestion;
			Question = textResponseQuestion.Question;
		}
	}
}
