﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2009, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
	This file is part of Calcium.

	Calcium is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Calcium is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Calcium.  If not, see http://www.gnu.org/licenses/.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2009-04-25 14:43:57Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;

namespace DanielVaughan.Services.Implementation
{
	public enum MessageImage
	{
		None,
		Asterisk,
		Error,
		Exclamation,
		Hand,
		Information,
		Question,
		Stop,
		Warning
	}

	public enum MessageButton
	{
		Ok = 0,
		OkCancel = 1,
		YesNo = 4,
		YesNoCancel = 3
	}

	public enum MessageResult
	{
		Cancel = 2,
		No = 7,
		None = 0,
		Ok = 1,
		Yes = 6
	}

	/// <summary>
	/// Base implementation of <see cref="IMessageService"/>.
	/// </summary>
	public abstract class MessageServiceBase : IMessageService
	{
		/*TODO: implement a means to modify the user MessageImportance threshold value. 
		 * This be done by subscribing to a composite property changed event. */
		public static MessageImportance MinumumImportance { get; protected set; }

		static MessageServiceBase()
		{
			MinumumImportance = MessageImportance.Low;
		}

		/// <summary>
		/// Shows a custom dialogue. Override this message in test mocks.
		/// This is the only place where a MessageBox is shown.
		/// </summary>
		/// <param name="message">The message text to display.</param>
		/// <param name="details">Can be <code>null</code>. 
		/// A detailed message that is display in a hidden area of the dialog, 
		/// and revealed on user request.</param>
		/// <param name="caption">The caption to display at the top of the dialog.</param>
		/// <param name="messageButton">The message box button enum value, 
		/// which determines what buttons are shown.</param>
		/// <param name="messageImage">The message box image, 
		/// which determines what icon is used.</param>
		/// <param name="importanceThreshold">The minimum filter level 
		/// that a user must have in order to receive the message.</param>
		/// <returns>The result of showing the message box.</returns>
		public abstract MessageResult ShowCustomDialog(
			string message, 
			string caption,
			MessageButton messageButton, 
			MessageImage messageImage,
			MessageImportance? importanceThreshold, 
			string details);


		public void ShowMessage(
			string message,
			string caption = null,
			MessageImportance importanceThreshold = MessageImportance.High,
			string details = null)
		{
			ShowCustomDialog(message, caption ?? "Message",
				MessageButton.Ok, 
				MessageImage.Information, 
				importanceThreshold, details);
		}


		public void ShowWarning(
			string message, string caption = null, string details = null)
		{
			ShowCustomDialog(message, caption ?? "Warning",
				MessageButton.Ok, MessageImage.Warning, null, details);
		}


		public void ShowError(
			string message, string caption = null, string details = null)
		{
			ShowCustomDialog(message, caption ?? "Error",
				MessageButton.Ok, MessageImage.Error, null, details);
		}


		public bool AskYesNoQuestion(
			string question, string caption = null, string details = null)
		{
			var messageBoxResult = ShowCustomDialog(question, caption ?? "Question",
				MessageButton.YesNo, MessageImage.Question, null, null);
			return messageBoxResult == MessageResult.Yes
				|| messageBoxResult == MessageResult.Ok;
		}

		public bool AskOkCancelQuestion(
			string question, string caption = null, string details = null)
		{
			var messageBoxResult = ShowCustomDialog(question, caption ?? "Question",
				MessageButton.OkCancel, MessageImage.Question, null, null);
			return messageBoxResult == MessageResult.Ok
				|| messageBoxResult == MessageResult.Yes;
		}

		public abstract void AskQuestion<TResponse>(
			IQuestion<TResponse> question, 
			Action<QuestionResult<TResponse>> resultHandler) 
				where TResponse : class, IQuestionResult;

		public YesNoCancelQuestionResult AskYesNoCancelQuestion(
			string question, string caption = null, string details = null)
		{
			var dialogResult = ShowCustomDialog(question, caption ?? "Question",
				MessageButton.YesNoCancel, MessageImage.Question, null, details);

			switch (dialogResult)
			{
				case MessageResult.Yes:
					return YesNoCancelQuestionResult.Yes;
				case MessageResult.No:
					return YesNoCancelQuestionResult.No;
				case MessageResult.Cancel:
					return YesNoCancelQuestionResult.Cancel;
				default:
					/* Should never get here. */
					throw new InvalidOperationException("Invalid dialog MessageBoxResult."); /* TODO: Make localizable resource. */
			}
		}
	}
}
