﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Device.Location;
using System.Windows.Input;

using DanielVaughan.Collections;
using DanielVaughan.Services;
using DanielVaughan.WindowsPhone7Unleashed.Examples.Bing.Route;
using DanielVaughan.WindowsPhone7Unleashed.Examples.BingMaps.Routing;
using DanielVaughan.WindowsPhone7Unleashed.GeoLocation;

using Microsoft.Phone.Controls.Maps;
using Microsoft.Phone.Controls.Maps.Core;
using Microsoft.Phone.Controls.Maps.Platform;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class BingMapsViewModel : ViewModelBase
	{
		readonly CredentialsProvider credentialsProvider
					= new ApplicationIdCredentialsProvider(App.BingMapsId);

		IRouteCalculator routeCalculator;

		public CredentialsProvider CredentialsProvider
		{
			get
			{
				return credentialsProvider;
			}
		}

		public BingMapsViewModel(IRouteCalculator routeCalculator)
		{
			this.routeCalculator = ArgumentValidator.AssertNotNull(
											routeCalculator, "routeCalculator");

			geoPositionWatcher = EnvironmentValues.UsingEmulator
					? new MockGeoCoordinateWatcher(2000)
					: (IGeoPositionWatcher<GeoCoordinate>)new GeoCoordinateWatcher();

			geoPositionWatcher.PositionChanged
				+= (o, args) =>
				       {
				           GeoCoordinate geoCoordinate = args.Position.Location;
				           Center = geoCoordinate;

				           Message = string.Format("{0} Lat, {1} Long", 
                               geoCoordinate.Latitude, geoCoordinate.Longitude);
				       };

			geoPositionWatcher.StatusChanged
				+= delegate(object o, GeoPositionStatusChangedEventArgs args)
				        {
					        GeoPositionStatus = args.Status;
				        };

			trackCommand = new DelegateCommand(obj => Start());

            toggleMapModeCommand = new DelegateCommand(obj => ToggleMapMode());

			addPushpinCommand = new DelegateCommand(obj => CreateNewPushPin());

			routeSearchCommand = new DelegateCommand(obj => Search());

			routeSearchToggleCommand = new DelegateCommand(
				obj => ToggleRouteSearchVisibility());

			itineraryToggleCommand = new DelegateCommand(
				obj => ToggleItineraryVisibility());
		}

	    void ToggleMapMode()
	    {
	        MapMode = mapMode is AerialMode 
                ? (MapMode)new RoadMode() : new AerialMode(true);
	    }

	    void Search()
	    {
	        if (!string.IsNullOrEmpty(fromAddress) && !string.IsNullOrEmpty(toAddress))
	        {
	            Message = "searching...";
	            routeCalculator.CalculateAsync(fromAddress, toAddress, HandleCalculationResult);
	        }
	        VisualState = HideItineraryState;
	        VisualState = HideRouteSearchState;
	    }

	    readonly IGeoPositionWatcher<GeoCoordinate> geoPositionWatcher;
		GeoPositionStatus geoPositionStatus = GeoPositionStatus.Initializing;

		public GeoPositionStatus GeoPositionStatus
		{
			get
			{
				return geoPositionStatus;
			}
			set
			{
				Assign(() => GeoPositionStatus, ref geoPositionStatus, value);
			}
		}

		bool tracking;
		readonly object startLock = new object();

		public bool Tracking
		{
			get
			{
				return tracking;
			}
		}

		void Start()
		{
			if (tracking)
			{
				return;
			}

			lock (startLock)
			{
				if (tracking)
				{
					return;
				}

				/* Before using geo location, it is a certification requirement 
				 * that you seek permission from the user first. 
				 * Calcium uses a settings service that makes it easy to store the response, 
				 * using the ISettingsService. */
				 
				bool canTrack = MessageService.AskYesNoQuestion(
					"Is it OK to use the geolocation system on your phone?");
				/*
				var settingsService = Dependency.Resolve<ISettingsService>();
				bool userConfirmedOk = settingsService.GetSetting(SettingKeys.UserConfirmedGeoLocationUsage, false);
				bool canTrack = false;
				if (userConfirmedOk)
				{
					canTrack = true;
				}
				else if (messageService.AskYesNoQuestion("Is it OK to use the GPS system on your phone?"))
				{
					settingsService.SetSetting(SettingKeys.UserConfirmedGeoLocationUsage, true);
					canTrack = true;
				} 
				*/
				if (!canTrack)
				{
					return;
				}

				tracking = true;
				OnPropertyChanged(() => Tracking);
				geoPositionWatcher.Start();
			}
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			set
			{
				Assign(() => Message, ref message, value);
			}
		}

		readonly DelegateCommand trackCommand;

		public ICommand TrackCommand
		{
			get
			{
				return trackCommand;
			}
		}

		readonly DelegateCommand toggleMapModeCommand;

		public ICommand ToggleMapModeCommand
		{
			get
			{
				return toggleMapModeCommand;
			}
		}

		public string MapModeCommandText
		{
			get
			{
				return mapMode is RoadMode ? "Aerial View" : "Road View";
			}
		}

	    MapMode mapMode = new AerialMode(true);

	    public MapMode MapMode
	    {
	        get
	        {
	            return mapMode;
	        }
	        set
	        {
	            Assign(() => MapMode, ref mapMode, value);
                OnPropertyChanged(() => MapModeCommandText);
	        }
	    }

		const double zoomLevelMaximum = 21;

		public double ZoomLevelMaximum
		{
			get
			{
				return zoomLevelMaximum;
			}
		}

		double zoomLevel = 14;

		public double ZoomLevel
		{
			get
			{
				return zoomLevel;
			}
			set
			{
				var newValue = Math.Max(1, Math.Min(zoomLevelMaximum, value));
				Assign(() => ZoomLevel, ref zoomLevel, newValue);
			}
		}

		GeoCoordinate center = new GeoCoordinate(45.8, 6.15);

		public GeoCoordinate Center
		{
			get
			{
				return center;
			}
			set
			{
				Assign("Center", ref center, value);
			}
		}

		#region PushPins

		readonly DelegateCommand addPushpinCommand;

		public ICommand AddPushpinCommand
		{
			get
			{
				return addPushpinCommand;
			}
		}

		void CreateNewPushPin()
		{
			var question = new TextResponseQuestion(
									"Please provide a name for the new push pin.");

			var messageService = Dependency.Resolve<IMessageService>();
			messageService.AskQuestion(question, 
				questionResult =>
			    {
					if (!questionResult.HasValidResponse)
					{
						return;
					}

					Random random = new Random();
					PushpinIcon randomIcon = (PushpinIcon)random.Next(0, 8);
					PushpinViewModel pushpin = new PushpinViewModel(
						questionResult.Response.Text, Center, randomIcon);

					pushpins.Add(pushpin);
			    });
		}

		readonly ObservableCollection<PushpinViewModel> pushpins 
						= new ObservableCollection<PushpinViewModel>();

		public IEnumerable<PushpinViewModel> Pushpins
		{
			get
			{
				return pushpins;
			}
		}

		#endregion

		#region Route Location

		/* TODO: replace. */
		string fromAddress = "Bellevue WA USA";

		public string FromAddress
		{
			get
			{
				return fromAddress;
			}
			set
			{
				Assign(() => FromAddress, ref fromAddress, value);
			}
		}

		/* TODO: replace. */
		string toAddress = "Microsoft Building 41";

		public string ToAddress
		{
			get
			{
				return toAddress;
			}
			set
			{
				Assign(() => ToAddress, ref toAddress, value);
			}
		}

		readonly DelegateCommand routeSearchCommand;

		public ICommand RouteSearchCommand
		{
			get
			{
				return routeSearchCommand;
			}
		}

		void HandleCalculationResult(RouteCalculationResult routeCalculationResult)
		{
			Message = string.Empty;
			if (routeCalculationResult.Error != null)
			{
				return;
			}

			ObservableCollection<MapItineraryItem> items = new ObservableCollection<MapItineraryItem>();
			RouteLeg routeLeg = routeCalculationResult.RouteResult.Legs[0];
			foreach (ItineraryItem itineraryItem in routeLeg.Itinerary)
			{
				items.Add(new MapItineraryItem(itineraryItem));
			}
			ItineraryItems = items;

			Routes.Clear();
			ObservableCollection<Location> routePoints 
				= routeCalculationResult.RouteResult.RoutePath.Points;
			Route route = new Route(routePoints);
			Routes = new ObservableCollection<Route> {route};
			LocationRect = LocationRect.CreateLocationRect(route.GeoCoordinates);
		}

		LocationRect locationRect;

		public LocationRect LocationRect
		{
			get
			{
				return locationRect;
			}
			private set
			{
				Assign(() => LocationRect, ref locationRect, value);
			}
		}

		IEnumerable<GeoCoordinate> routeCoordinates = new ObservableCollection<GeoCoordinate>();

		public IEnumerable<GeoCoordinate> RouteCoordinates
		{
			get
			{
				return routeCoordinates;
			}
			private set
			{
				Assign(() => RouteCoordinates, ref routeCoordinates, value);
			}
		}

		ObservableCollection<Route> routes = new ObservableCollection<Route>();

		public ObservableCollection<Route> Routes
		{
			get
			{
				return routes;
			}
			private set
			{
				Assign(() => Routes, ref routes, value);
			}
		}

		ObservableCollection<MapItineraryItem> itineraryItems = new ObservableCollection<MapItineraryItem>();

		public ObservableCollection<MapItineraryItem> ItineraryItems
		{
			get
			{
				return itineraryItems;
			}
			private set
			{
				Assign(() => ItineraryItems, ref itineraryItems, value);
			}
		}

		readonly DelegateCommand routeSearchToggleCommand;

		public ICommand RouteSearchToggleCommand
		{
			get
			{
				return routeSearchToggleCommand;
			}
		}

		readonly DelegateCommand itineraryToggleCommand;

		public ICommand ItineraryToggleCommand
		{
			get
			{
				return itineraryToggleCommand;
			}
		}

		#endregion

		#region Visual States
		public const string ShowRouteSearchState = "ShowRouteSearch";
		public const string HideRouteSearchState = "HideRouteSearch";
		string routeSearchVisualState = HideRouteSearchState;
		public const string ShowItineraryState = "ShowItinerary";
		public const string HideItineraryState = "HideItinerary";
		string itineraryVisualState = HideItineraryState;

	    string visualState = HideItineraryState;

	    public string VisualState
	    {
	        get
	        {
	            return visualState;
	        }
	        private set
	        {
	            Assign(() => VisualState, ref visualState, value);
	        }
	    }

        void ToggleRouteSearchVisibility()
        {
            if (routeSearchVisualState == ShowRouteSearchState)
            {
                VisualState = routeSearchVisualState = HideRouteSearchState;
            }
            else
            {
                VisualState = routeSearchVisualState = ShowRouteSearchState;
            }
        }

        void ToggleItineraryVisibility()
        {
            if (itineraryVisualState == ShowItineraryState)
            {
                VisualState = itineraryVisualState = HideItineraryState;
            }
            else
            {
                VisualState = itineraryVisualState = ShowItineraryState;
            }
        }
		#endregion
	}
}
