﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Threading;

using DanielVaughan.WindowsPhone7Unleashed.Examples.Bing.Geocode;
using DanielVaughan.WindowsPhone7Unleashed.Examples.Bing.Route;

using Microsoft.Phone.Controls.Maps;
using Microsoft.Phone.Controls.Maps.Platform;
using Microsoft.Phone.Reactive;

using Confidence = DanielVaughan.WindowsPhone7Unleashed.Examples.Bing.Geocode.Confidence;
using ExecutionOptions = DanielVaughan.WindowsPhone7Unleashed.Examples.Bing.Route.ExecutionOptions;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.BingMaps.Routing
{
	public class RouteCalculator : IRouteCalculator
	{
		readonly CredentialsProvider credentialsProvider;
		readonly GeocodeServiceClient geocodeClient = new GeocodeServiceClient();
		readonly RouteServiceClient routeClient = new RouteServiceClient();
		
		public RouteCalculator(CredentialsProvider credentialsProvider)
		{
			this.credentialsProvider = ArgumentValidator.AssertNotNull(
										credentialsProvider, "credentialsProvider");
		}

		public void CalculateAsync(
			string fromAddress,
			string toAddress,
			Action<RouteCalculationResult> completeAction)
		{
			ArgumentValidator.AssertNotNullOrEmpty(fromAddress, "fromAddress");
			ArgumentValidator.AssertNotNullOrEmpty(toAddress, "toAddress");
			ArgumentValidator.AssertNotNull(completeAction, "completeAction");

			credentialsProvider.GetCredentials(
				credentials => CalculateAsyncCore(
					credentials, fromAddress, toAddress, completeAction));
		}

		void CalculateAsyncCore(
				Credentials credentials, 
				string fromAddress,
				string toAddress,
				Action<RouteCalculationResult> completeAction)
		{
			IObservable<IEvent<GeocodeCompletedEventArgs>> geocodingCompleted 
				= Observable.FromEvent<GeocodeCompletedEventArgs>
					(h => geocodeClient.GeocodeCompleted += h,
					h => geocodeClient.GeocodeCompleted -= h).Take(2);

			var geocodeEvents = geocodingCompleted.TakeLast(2)
				.Let(observable => observable.Zip(observable.Skip(1), 
					(previous, current) => new
					                       	{
					                       		Args1 = previous.EventArgs, 
												Args2 = current.EventArgs
					                       	})); 

			geocodeEvents.Subscribe(
				x =>
			    {
					RouteCalculationError error 
                        = RouteCalculationError.CreateErrorIfInvalid(x.Args1) 
							?? RouteCalculationError.CreateErrorIfInvalid(x.Args2);

					if (error != null)
					{
						HandleRouteCalculated(
							new RouteCalculationResult(null, error), completeAction);
						return;
					}

					var request = new RouteRequest
						{
							Culture = CultureInfo.CurrentUICulture.Name,
							Waypoints = new ObservableCollection<Waypoint>(),
							/* Suppress exceptions. */
							ExecutionOptions = new ExecutionOptions
							                   	{
							                   		SuppressFaults = true
							                   	},
							Options = new RouteOptions
							          	{
							          		RoutePathType = RoutePathType.Points
							          	}
						};

					request.Waypoints.Add(ToWaypoint(x.Args1.Result.Results.First()));
					request.Waypoints.Add(ToWaypoint(x.Args2.Result.Results.First()));
					request.Credentials = credentials;
					routeClient.CalculateRouteAsync(request);
			    });

			var routeEvent = Observable.FromEvent<CalculateRouteCompletedEventArgs>
					 (h => routeClient.CalculateRouteCompleted += h,
					  h => routeClient.CalculateRouteCompleted -= h).Take(1);

			routeEvent.Subscribe(x => HandleRouteCalculated(
				new RouteCalculationResult(x.EventArgs.Result.Result,
						RouteCalculationError.CreateErrorIfInvalid(x.EventArgs)),
						completeAction));

			geocodeClient.GeocodeAsync(CreateGeocodeRequest(toAddress, credentials));
			geocodeClient.GeocodeAsync(CreateGeocodeRequest(fromAddress, credentials));
		}

		GeocodeRequest CreateGeocodeRequest(string address, Credentials credentials)
		{
			GeocodeRequest geocodeRequest = new GeocodeRequest
				{
					Culture = CultureInfo.CurrentUICulture.Name,
					Query = address,
					/* Suppress exceptions. */
					ExecutionOptions = new Bing.Geocode.ExecutionOptions
				                   		{
				                   			SuppressFaults = true
				                   		},
					/* Only accept results with high confidence. */
					Options = new GeocodeOptions { 
									Filters = new ObservableCollection<FilterBase> {
				          		          			new ConfidenceFilter {
				          		          					MinimumConfidence = Confidence.High }}},
									Credentials = credentials
				};
			return geocodeRequest;
		}

		protected virtual void HandleRouteCalculated(
			RouteCalculationResult calculationResult, 
			Action<RouteCalculationResult> completeAction)
		{
			ArgumentValidator.AssertNotNull(calculationResult, "calculationResult");
			Deployment.Current.Dispatcher.InvokeIfRequired(
									() => completeAction(calculationResult));
		}

		static Waypoint ToWaypoint(GeocodeResult result)
		{
			return new Waypoint {
							Description = result.DisplayName, 
							Location = new Location
								{
									Latitude = result.Locations[0].Latitude, 
									Longitude = result.Locations[0].Longitude
								}};
		}

	}
}
