﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Input;

using DanielVaughan.Collections;
using DanielVaughan.WindowsPhone7Unleashed.Ebay;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class EbaySearchStateViewModel : ViewModelBase, IEbaySearchViewModel
	{
		readonly INetworkConnectionMonitor networkConnectionMonitor;
		readonly Func<IEbayClient> getEbayClientFunc;

		public EbaySearchStateViewModel(
			Func<IEbayClient> getEbayClientFunc, 
			INetworkConnectionMonitor networkConnectionMonitor) 
		{
			this.networkConnectionMonitor = ArgumentValidator.AssertNotNull(
						  networkConnectionMonitor, "networkConnectionMonitor");
			this.getEbayClientFunc = ArgumentValidator.AssertNotNull(
										getEbayClientFunc, "getEbayClientFunc");

			networkConnectionMonitor.NetworkConnectionChanged 
				+= (sender, args) => UpdateCommands();
			
			SearchText = "star wars action figure";

			searchCommand = new DelegateCommand<string>(
				query =>
				{
					if (!string.IsNullOrEmpty(query))
					{
						Search(query);
					}
				}, obj => networkConnectionMonitor.Connected);

			fetchMoreDataCommand = new DelegateCommand(
				obj =>
				{
					if (!string.IsNullOrEmpty(lastQuery))
					{
						Search(lastQuery, true);
					}
				}, obj => networkConnectionMonitor.Connected);

			
			RegisterStatefulProperty(ApplicationStateType.Transient, () => SearchText);
			RegisterStatefulProperty(ApplicationStateType.Transient, () => Items, x => Items = x);

			RegisterState("lastQuery", () => lastQuery, x => lastQuery = x, ApplicationStateType.Transient);
			//RegisterStatefulProperty(ApplicationStateType.Persistent, () => SearchCount);
		}

		void UpdateCommands()
		{
			searchCommand.RaiseCanExecuteChanged();
			fetchMoreDataCommand.RaiseCanExecuteChanged();
		}

		const int chunkSize = 10;
		string lastQuery;
		bool appendResult;
		
		void Search(string query, bool append = false)
		{
			if (lastQuery == query && Busy)
			{
				return;
			}

			lastQuery = query;
			appendResult = append;

			var ebayClient = getEbayClientFunc();
			ebayClient.SearchItemsComplete -= OnEbayClientSearchItemsComplete;
			ebayClient.SearchItemsComplete += OnEbayClientSearchItemsComplete;
			int beginIndex = appendResult ? Items.Count : 0;

			try
			{
				Busy = true;
				ebayClient.SearchItemsAsync(query, beginIndex, chunkSize);
				SearchCount++;
			}
			catch (Exception ex)
			{
				Busy = false;
				Console.WriteLine("Unable to perform search." + ex);
				MessageService.ShowError("Unable to perform search.");
			}
		}

		void OnEbayClientSearchItemsComplete(
			object sender, SearchItemsCompleteEventArgs args)
		{
			/* Ignore old query results. */
			if (args.Query != lastQuery)
			{
				return;
			}

			Busy = false;
			if (args.Error == null)
			{
				if (args.Result.Count > 0)
				{
					if (appendResult)
					{
						//Items.AddRange(args.Result);
						foreach (var item in args.Result)
						{
							Items.Add(item);
						}
					}
					else
					{
						Items = new ObservableCollection<Item>(args.Result);
					}
				}
				else
				{
					MessageService.ShowMessage("No match found.");
				}
			}
			else
			{
				MessageService.ShowError(
					"An error occured while attempting to search.");
			}
		}

		bool busy;

		public bool Busy
		{
			get
			{
				return busy;
			}
			private set
			{
				Assign(() => Busy, ref busy, value);
			}
		}

		string searchText;

		public string SearchText
		{
			get
			{
				return searchText;
			}
			set
			{
				Assign(() => SearchText, ref searchText, value);
			}
		}

		[Stateful(ApplicationStateType.Persistent)]
		public int SearchCount { get; set; }

		ObservableCollection<Item> items = new ObservableCollection<Item>();

		public ObservableCollection<Item> Items
		{
			get
			{
				return items;
			}
			private set
			{
				Assign(() => Items, ref items, value);
			}
		}

		readonly DelegateCommand<string> searchCommand;

		public ICommand SearchCommand
		{
			get
			{
				return searchCommand;
			}
		}

		readonly DelegateCommand fetchMoreDataCommand;

		public ICommand FetchMoreDataCommand
		{
			get
			{
				return fetchMoreDataCommand;
			}
		}

//		const string itemsStateKey = "EbayMainViewModel_Items";
//		const string queryStateKey = "EbayMainViewModel_Query";
//
//		public void LoadState(
//			IDictionary<string, object> persistentDictionary, 
//			IDictionary<string, object> transientDictionary)
//		{
//			base.LoadState(persistentDictionary, transientDictionary);
//			object state;
//			if (transientDictionary.TryGetValue(itemsStateKey, out state))
//			{
//				byte[] bytes = state as byte[];
//				Items = Deserialize<ObservableList<Item>>(bytes);
//			}
//
//			if (transientDictionary.TryGetValue(queryStateKey, out state))
//			{
//				lastQuery = (string)state;
//				SearchText = lastQuery;
//			}
//		}
//
//		public void SaveState(
//			IDictionary<string, object> persistentDictionary,
//			IDictionary<string, object> transientDictionary)
//		{
//			base.SaveState(persistentDictionary, transientDictionary);
//			byte[] state = Serialize(items);
//			transientDictionary[itemsStateKey] = state;
//			transientDictionary[queryStateKey] = lastQuery;
//		}
	}
}
