﻿using System;
using System.Collections.Generic;
using System.Data.Linq;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Threading;
using System.Xml.Linq;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class TwitterService : ITwitterService
	{
		readonly TwitterDatabaseUtility twitterDatabaseUtility = new TwitterDatabaseUtility();
		readonly TwitterDataContext dataContext;

		public TwitterService()
		{
			dataContext = twitterDatabaseUtility.CreateContext();
			//dataContext.ObjectTrackingEnabled = false;
			//dataContext.TimelineItems.Context.DeferredLoadingEnabled = false;
		}

		public void GetCachedTimeline(
			string screenName, Action<ResultEventArgs<TwitterUser>> completeAction)
		{
			ArgumentValidator.AssertNotNullOrWhiteSpace(screenName, "userName");
			ArgumentValidator.AssertNotNull(completeAction, "completeAction");

			ThreadPool.QueueUserWorkItem(
				delegate
				{
					try
					{
						GetCachedTimelineCore(screenName, completeAction);
					}
					catch (Exception ex)
					{
						Console.WriteLine(ex);
						completeAction(new ResultEventArgs<TwitterUser>(null, ex));
					}
				});
		}

		void GetCachedTimelineCore(
				string userName, Action<ResultEventArgs<TwitterUser>> completeAction)
		{
			TwitterUser storedUser = dataContext.TwitterUsers.Where(
					user => user.ScreenName == userName).FirstOrDefault();

			completeAction(new ResultEventArgs<TwitterUser>(storedUser));
		}

		public void GetTimeline(
			string screenName, Action<ResultEventArgs<TwitterUser>> completeAction)
		{
			ArgumentValidator.AssertNotNullOrWhiteSpace(screenName, "userName");
			ArgumentValidator.AssertNotNull(completeAction, "completeAction");

			ThreadPool.QueueUserWorkItem(
				delegate
			    {
			        try
			        {
						/* Wait for a moment to demonstrate the database retrieval. */
						Wait(3000); /* Equivalent to Thread.Sleep(3000); */
			            GetTimelineCore(screenName, completeAction);
			        }
			        catch (Exception ex)
			        {
			            Console.WriteLine(ex);
						completeAction(new ResultEventArgs<TwitterUser>(null, ex));
			        }
			    });
		}

		void GetTimelineCore(
				string screenName, Action<ResultEventArgs<TwitterUser>> completeAction)
		{
			WebClient twitterClient = new WebClient();
			twitterClient.DownloadStringCompleted 
				+= (o, args) =>
				{
					if (args.Error != null)
					{
						completeAction(new ResultEventArgs<TwitterUser>(null, args.Error));
						return;
					}

					try
					{
						XElement rootElement = XElement.Parse(args.Result);

						XElement element = rootElement.Descendants("status").First();
						XElement userElement = element.Descendants("user").First();
						var twitterUser = new TwitterUser
											{
												Id = userElement.Element("id").Value,
												ScreenName = userElement.Element("screen_name").Value,
												Description = userElement.Element("description").Value,
												ImageUrl = userElement.Element("profile_image_url").Value,
											};

						IEnumerable<TimelineItem> items = from statusElement in rootElement.Descendants("status")
									select new TimelineItem
									{
										Id = statusElement.Element("id").Value,
										ReceivedTime = ConvertToDateTime(
											statusElement.Element("created_at").Value),
										Text = statusElement.Element("text").Value
									};

						TwitterUser storedUser = dataContext.TwitterUsers.Where(
							user => user.ScreenName == screenName).FirstOrDefault();

						if (storedUser != null)
						{
							dataContext.TimelineItems.DeleteAllOnSubmit(
								dataContext.TimelineItems.Where(
									item => item.TwitterUserId == storedUser.Id));
							dataContext.TwitterUsers.DeleteOnSubmit(storedUser);
							dataContext.SubmitChanges();
						}

						twitterUser.TimelineItems.AddRange(items);
						dataContext.TwitterUsers.InsertOnSubmit(twitterUser);

						/* This raises an exception if using a database located in the XAP. */
						dataContext.SubmitChanges();

						completeAction(new ResultEventArgs<TwitterUser>(twitterUser));
					}
					catch (Exception ex)
					{
						Debug.WriteLine("Unable to get timeline. " + ex);
						completeAction(new ResultEventArgs<TwitterUser>(null, ex));
						return;
					}
				};

			twitterClient.DownloadStringAsync(
				new Uri("http://api.twitter.com/1/statuses/user_timeline.xml?screen_name=" + screenName));
		}

		static DateTime ConvertToDateTime(string twitterDate)
		{
			DateTime dateTime = DateTime.ParseExact(
				twitterDate, "ddd MMM dd HH:mm:ss zzz yyyy", CultureInfo.InvariantCulture);
			return dateTime;
		}

		static readonly object waitLock = new object();

		static void Wait(int delayMs)
		{
			if (delayMs > 0)
			{
				lock (waitLock)
				{
					Monitor.Wait(waitLock, delayMs);
				}
			}
		}
	}
}
