﻿using System.Device.Location;
using System.Windows.Input;

using DanielVaughan.WindowsPhone7Unleashed.GeoLocation;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class GeoPositionViewModel : ViewModelBase
	{
		IGeoPositionWatcher<GeoCoordinate> geoPositionWatcher;
		GeoPositionSampler<GeoCoordinate> sampler;

		public GeoPositionViewModel()
		{
			startCommand = new DelegateCommand(obj => Start(), arg => CanStart);
			stopCommand = new DelegateCommand(obj => Stop(), arg => Running);
			PropertyChanged += delegate { RefreshCommands(); };
		}

		void RefreshCommands()
		{
			startCommand.RaiseCanExecuteChanged();
			stopCommand.RaiseCanExecuteChanged();
		}

		GeoCoordinate geoCoordinate;

		public GeoCoordinate GeoCoordinate
		{
			get
			{
				return geoCoordinate;
			}
			private set
			{
				Assign("GeoCoordinate", ref geoCoordinate, value);
			}
		}

		GeoPositionStatus geoPositionStatus = GeoPositionStatus.Initializing;

		public GeoPositionStatus GeoPositionStatus
		{
			get
			{
				return geoPositionStatus;
			}
			private set
			{
			    Assign(() => GeoPositionStatus, ref geoPositionStatus, value);
			}
		}

		int sampleIntervalMs;

		public int SampleIntervalMs
		{
			get
			{
				return sampleIntervalMs;
			}
			set
			{
				ArgumentValidator.AssertGreaterThanOrEqualTo(0, value, "value");
			    Assign(() => SampleIntervalMs, ref sampleIntervalMs, value);
			}
		}

		bool canStart = true;

		public bool CanStart
		{
			get
			{
				return canStart;
			}
			private set
			{
			    Assign(() => CanStart, ref canStart, value);
			}
		}

		bool running;

		public bool Running
		{
			get
			{
				return running;
			}
			private set
			{
			    Assign(() => Running, ref running, value);
			}
		}

		void Start()
		{
			if (running)
			{
				return;
			}

			Running = true;
			CanStart = false;

			geoPositionWatcher = EnvironmentValues.UsingEmulator
				? new MockGeoCoordinateWatcher()
				: (IGeoPositionWatcher<GeoCoordinate>)new GeoCoordinateWatcher 
														{MovementThreshold = 20};

			if (sampleIntervalMs > 0)
			{
				sampler = new GeoPositionSampler<GeoCoordinate>(
						            geoPositionWatcher, sampleIntervalMs);

				sampler.PropertyChanged += (o, args) =>
				                           	{
				                           		ResolveAddress(sampler.Location);
				                           		GeoCoordinate = sampler.Location;
				                           		GeoPositionStatus = sampler.Status;
				                           	};
			}
			else
			{
				geoPositionWatcher.PositionChanged
					+= (o, args) =>
					   	{
					   		GeoCoordinate = args.Position.Location;
							ResolveAddress(args.Position.Location);
					   	};

				geoPositionWatcher.StatusChanged 
					+= (o, args) => GeoPositionStatus = args.Status;
			}

			geoPositionWatcher.Start();
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			set
			{
			    Assign(() => Message, ref message, value);
			}
		}	

		void ResolveAddress(GeoCoordinate geoCoordinate)
		{
			if (geoCoordinate == null || geoCoordinate.IsUnknown)
			{
				/* If the geoCoordinate has not been assigned, 
				 * i.e., its Latitude or Longitude is NaN, 
				 * then it will raise an argument exception 
				 * when ResolveAddresAsync is called. */
				return;
			}

			CivicAddressResolver resolver = new CivicAddressResolver();
			resolver.ResolveAddressCompleted += (o, args) =>
					{
						if (args.Error != null)
						{
							Message = "Unable to resolve address. " 
								+ args.Error.ToString();
						} 
						else if (args.Cancelled)
						{
							Message = "Retrieving address cancelled.";
						}
						else if (args.Address == null)
						{
							Message = "Address is null.";
						}
						else if (args.Address.IsUnknown)
						{
							Message = "Unknown address.";
						}
						else
						{
							Message = string.Format("City= {0}", args.Address.City);
						}
					};
			
			resolver.ResolveAddressAsync(geoCoordinate);
		}

		void Stop()
		{
			if (!running || geoPositionWatcher == null)
			{
				return;
			}
			
			if (sampler != null)
			{
				sampler.Dispose();
			}

			geoPositionWatcher.Stop();
			Running = false;
			CanStart = true;
		}

		readonly DelegateCommand startCommand;

		public ICommand StartCommand
		{
			get
			{
				return startCommand;
			}
		}

		readonly DelegateCommand stopCommand;

		public ICommand StopCommand
		{
			get
			{
				return stopCommand;
			}
		}
	}
}
