﻿using System;
using System.Collections.Generic;
using System.Windows.Input;

using DanielVaughan.WindowsPhone7Unleashed.InputValidation;
using DanielVaughan.WindowsPhone7Unleashed.LauncherModel;
using DanielVaughan.WindowsPhone7Unleashed.Launchers;

using Microsoft.Phone.Tasks;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class SendEmailViewModel : ViewModelBase
	{
		readonly Func<ChooserAdapter<EmailResult>> getAddressAdapterAction;
		readonly Func<EmailComposeTaskAdapter> getEmailTaskAdapter;
		bool sendAttempted;
		bool sending;

		public SendEmailViewModel(
			Func<ChooserAdapter<EmailResult>> getAddressAdapterAction,
			Func<EmailComposeTaskAdapter> getEmailTaskAdapter)
		{
			this.getAddressAdapterAction = ArgumentValidator.AssertNotNull(
							getAddressAdapterAction, "getAddressAdapterAction");
			this.getEmailTaskAdapter = ArgumentValidator.AssertNotNull(
							getEmailTaskAdapter, "getEmailTaskAdapter");;

			/* Add properties that are to be validated. */
			AddValidationProperty(() => To);
			AddValidationProperty(() => Body);
			AddValidationProperty(() => Cc);
			AddValidationProperty(() => Subject);

            sendEmailCommand = new DelegateCommand(arg => SendEmail(), 
                arg => !sending && (!sendAttempted || !HasErrors));

			selectAddressCommand = new DelegateCommand<string>(SelectAddress);
			PropertyChanged += (sender, args) => RefreshCommands();
		}

        void SendEmail()
        {
            if (sending)
            {
                return;
            }

            try
            {
                sending = sendAttempted = true;
                Message = string.Empty;
                RefreshCommands();
                IsComplete(() =>
                           {
                               sending = false;
                               EmailComposeTaskAdapter task = getEmailTaskAdapter();
                               task.Body = body;
                               task.Cc = cc;
                               task.Subject = subject;
                               task.To = to;
                               task.Show();
                           },
                           () =>
                           {
                               sending = false;
                               RefreshCommands();
                           },
                           obj =>
                           {
                               sending = false;
                               RefreshCommands();
                               string errorMessage = obj != null
                                           ? obj.Message : string.Empty;
                               Message = "Unable to send email: "
                                         + errorMessage;
                           });
            }
            catch (Exception ex)
            {
                sending = false;
                Message = "Unable to send email: " + ex.Message;
            }
        }

		/// <summary>
		/// A call to RefreshCommands raises the CanExecuteChanged event of the command, 
		/// which in turn sets the IsEnabled property of the Button.
		/// </summary>
		void RefreshCommands()
		{
			sendEmailCommand.RaiseCanExecuteChanged();
		}

		void SelectAddress(string propertyToAssign)
		{
			ChooserAdapter<EmailResult> emailAddressChooserTask 
												= getAddressAdapterAction();
			if (emailAddressChooserTask == null)
			{
				throw new InvalidOperationException(
					"ChooserAdapter<EmailResult> should not be null.");
			}

			emailAddressChooserTask.Completed += new EventHandler<EmailResult>(
				(sender, args) =>
			    {
			        if (args.Error != null)
			        {
			            return;
			        }
			        string assignValue;
			        if (args.TaskResult == TaskResult.OK)
			        {
			            assignValue = args.Email;
			        }
			        else if (args.TaskResult == TaskResult.None)
			        {
			            assignValue = string.Empty;
			        }
			        else
			        {
			            return;
			        }

			        if (propertyToAssign == "To")
			        {
			            To = assignValue;
			        }
			        else if (propertyToAssign == "Cc")
			        {
			            Cc = assignValue;
			        }
			        else
			        {
			            throw new ArgumentOutOfRangeException("CommandParameter", 
							"Argument must be either To or Cc");
			        }
			    });
			emailAddressChooserTask.Show();
		}

		protected override IEnumerable<DataValidationError> GetPropertyErrors(
			string propertyName, object value)
		{
			if (propertyName == "Cc")
			{
				yield break;
			}

			if (string.IsNullOrEmpty((string)value))
			{
				int errorId = GetMissingPropertyErrorId(propertyName);
				string errorMessage 
					= string.Format("{0} is a required field.", propertyName);
				yield return new DataValidationError(errorId, errorMessage);
			}
		}

		int GetMissingPropertyErrorId(string propertyName)
		{
			switch (propertyName)
			{
				case "To":
					return 1;
				case "Body":
					return 2;
				case "Cc":
					return 3;
				case "Subject":
					return 4;
				default:
					throw new ArgumentOutOfRangeException("propertyName", 
						"Unknown propertyName " + propertyName);
			}
		}

		#region SelectAddressCommand property and field

		readonly DelegateCommand<string> selectAddressCommand;

		public ICommand SelectAddressCommand
		{
			get
			{
				return selectAddressCommand;
			}
		}

		#endregion
		
		#region SendEmailCommand property and field

		readonly DelegateCommand<object> sendEmailCommand;

		public ICommand SendEmailCommand
		{
			get
			{
				return sendEmailCommand;
			}
		}

		#endregion

		#region To property and field

		string to;

		public string To
		{
			get
			{
				return to;
			}
			set
			{
				Assign("To", ref to, value);
			}
		}

		#endregion

		#region Body property and field

		string body;

		public string Body
		{
			get
			{
				return body;
			}
			set
			{
				Assign("Body", ref body, value);
			}
		}

		#endregion

		#region Cc property and field

		string cc;

		public string Cc
		{
			get
			{
				return cc;
			}
			set
			{
				Assign("Cc", ref cc, value);
			}
		}

		#endregion

		#region Subject property and field

		string subject;

		public string Subject
		{
			get
			{
				return subject;
			}
			set
			{
				Assign("Subject", ref subject, value);
			}
		}

		#endregion

		#region Message property and field

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			private set
			{
				Assign("Message", ref message, value);
			}
		}

		#endregion
	}
}
