﻿// (c) Copyright Microsoft Corporation.
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.

using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Windows;
using System.Windows.Resources;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	/// <summary>
	/// A class to return paragraphs of random sentences or a word list from lorem ipsum data.
	/// </summary>
	public class LoremIpsumGenerator : IEnumerable<string>
	{
		public enum Capitalization
		{
			None,
			FirstWord,
			AllWords
		}

		static readonly Random random = new Random(42);

		static readonly StringBuilder stringBuilder = new StringBuilder();
		static List<string> sentences;
		static List<string> words;

		/// <summary>
		/// Returns random lorem ipsum sentences combined into a single string.
		/// </summary>
		/// <param name="sentenceCount">The nunmber of sentences.</param>
		/// <returns>The paragraph, composed of random sentences.</returns>
		public static string GetParagraph(int sentenceCount)
		{
			EnsureSentences();

			stringBuilder.Length = 0;

			while (sentenceCount-- > 0)
			{
				stringBuilder.Append(random.Next(sentences));
				if (sentenceCount > 0)
				{
					stringBuilder.Append(' ');
				}
			}

			return stringBuilder.ToString();
		}

		/// <summary>
		/// Return an alphabetized, lower-case list of lorem ipsum words.
		/// </summary>
		public static ICollection<string> Words
		{
			get
			{
				EnsureWords();
				return words;
			}
		}

		/// <summary>
		/// Get a string composed of random lorem ipsum words. Will not end with punctuation.
		/// </summary>
		/// <param name="wordCount">Number of words.</param>
		/// <param name="capitalize">How to capitalize the words.</param>
		/// <returns></returns>
		public static string GetWords(int wordCount, Capitalization capitalization)
		{
			EnsureWords();

			stringBuilder.Length = 0;

			while (wordCount-- > 0)
			{
				int position = stringBuilder.Length;
				stringBuilder.Append(random.Next(words));
				if (capitalization == Capitalization.AllWords || (position == 0 && capitalization == Capitalization.FirstWord))
				{
					stringBuilder[position] = char.ToUpper(stringBuilder[position]);
				}

				if (wordCount > 0)
				{
					stringBuilder.Append(' ');
				}
			}

			return stringBuilder.ToString();
		}

		/// <summary>
		/// Enumerates the Words property.
		/// </summary>
		/// <returns>The enumerator.</returns>
		public IEnumerator<string> GetEnumerator()
		{
			return Words.GetEnumerator();
		}

		/// <summary>
		/// Enumerates the Words property.
		/// </summary>
		/// <returns>The enumerator.</returns>
		IEnumerator IEnumerable.GetEnumerator()
		{
			return Words.GetEnumerator();
		}

		/// <summary>
		/// Reads the lorem ipsum sentences. Supplies some data in case reading fails, which
		/// it will do at design time.
		/// </summary>
		static void EnsureSentences()
		{
			if (sentences == null)
			{
				sentences = new List<string>();

				StreamResourceInfo info = Application.GetResourceStream(new Uri("Data/LoremIpsum.txt", UriKind.Relative));
				if (info != null)
				{
					Stream stream = info.Stream;
					if (stream != null)
					{
						using (var reader = new StreamReader(stream))
						{
							while (!reader.EndOfStream)
							{
								sentences.Add(reader.ReadLine());
							}
						}
						;
					}
				}

				if (sentences.Count == 0)
				{
					sentences.Add("Lorem ipsum dolor sit amet, consectetuer adipiscing elit.");
					sentences.Add("Maecenas porttitor congue massa.");
					sentences.Add("Fusce posuere, magna sed pulvinar ultricies, purus lectus malesuada libero, sit amet commodo magna eros quis urna.");
					sentences.Add("Nunc viverra imperdiet enim.");
					sentences.Add("Fusce est.");
				}
			}
		}

		/// <summary>
		/// Creates an alphabetized list of the words from the lorem ipsum text.
		/// </summary>
		static void EnsureWords()
		{
			char[] separators = {' ', ',', '.'};

			EnsureSentences();

			if (words == null)
			{
				var temp = new Dictionary<string, object>();

				foreach (string sentence in sentences)
				{
					string[] words2 = sentence.Split(
						separators, StringSplitOptions.RemoveEmptyEntries);
					foreach (string word in words2)
					{
						temp[word.ToLower()] = null;
					}
				}

				words = new List<string>(temp.Keys);
				words.Sort();
			}
		}
	}

	public static class RandomExtensions
	{
		/// <summary>
		/// Return a random item from a list.
		/// </summary>
		/// <typeparam name="T">The item type.</typeparam>
		/// <param name="random">The Random instance.</param>
		/// <param name="list">The list to choose from.</param>
		/// <returns>A randomly selected item from the list.</returns>
		public static T Next<T>(this Random random, IList<T> list)
		{
			return list[random.Next(list.Count)];
		}
	}
}
