﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.IO.IsolatedStorage;
using System.Runtime.Serialization.Json;

using DanielVaughan.WindowsPhone7Unleashed.Examples.StockQuoteServiceReference;
using DanielVaughan.WindowsPhone7Unleashed.PushNotification;
using DanielVaughan.WindowsPhone7Unleashed.Services;

using Microsoft.Phone.Notification;
using Microsoft.Phone.Shell;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class StockQuoterSettings
	{
		public string StockSymbol { get; set; }
	}

	public enum StockPriceDirection
	{
		None,
		Up,
		Down
	}

	public class PushNotificationViewModel : ViewModelBase
	{
		const string channelName = "StockChannel";
		const string settingsKey = "StockQuoterSettings";
		StockQuoterSettings settings;
		bool warmResume;

		public PushNotificationViewModel()
		{
			RestoreState();

			PhoneApplicationService.Current.Deactivated += delegate 
												{
													Debug.WriteLine("Deactivated");
			                                     	warmResume = true; 
													SaveState(); 
												};

			PhoneApplicationService.Current.Activated += delegate
			                             	{
												Debug.WriteLine("Activated");
												if (warmResume)
												{
													return;
												}
			                             		RestoreState();
			                             	};
		}

		void RestoreState()
		{
			if (!IsolatedStorageSettings.ApplicationSettings.TryGetValue(
					settingsKey, out settings))
			{
				/* Settings was not previously stored in isolated storage, 
				 * so create a new object initialized with the default values. */
				settings = new StockQuoterSettings
				{
					StockSymbol = "MSFT"
				};
			}
			stockSymbol = settings.StockSymbol;
			Subscribe();
		}

		public void SaveState()
		{
			IsolatedStorageSettings.ApplicationSettings[settingsKey] = settings;
			IsolatedStorageSettings.ApplicationSettings.Save();
		}

		Uri channelUri;

		public Uri ChannelUri
		{
			get
			{
				return channelUri;
			}
			private set
			{
				Assign("ChannelUri", ref channelUri, value);
			}
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			private set
			{
				Assign("Message", ref message, value);
			}
		}

		string stockSymbol;

		public string StockSymbol
		{
			get
			{
				return stockSymbol;
			}
			set
			{
				Assign("StockCode", ref stockSymbol, value);
			}
		}

		PushNotificationSubscriber subscriber;

		void InitializeSubscriber()
		{
			subscriber = new PushNotificationSubscriber(
								channelName, null, RegisterWithCloudService);
			subscriber.HttpNotificationReceived += subscriber_HttpNotificationReceived;
			subscriber.ChannelUriUpdated += subscriber_ChannelUriUpdated;
			subscriber.ErrorOccurred += subscriber_ErrorOccurred;
		}

		public class SubscriberSettings
		{
			public string UserAllowedSubscription { get; set; }
		}

		void Subscribe()
		{
			if (subscriber == null)
			{
				InitializeSubscriber();
			}
			subscriber.Subscribe();
		}

		void subscriber_HttpNotificationReceived(
			object sender, HttpNotificationEventArgs e)
		{
			Message = "Raw notification received.";
			Stream bodyStream = e.Notification.Body;

			if (bodyStream != null)
			{
				DataContractJsonSerializer serializer
						= new DataContractJsonSerializer(typeof(StockQuote));

				StockQuote = (StockQuote)serializer.ReadObject(bodyStream);
			}
		}

		StockQuote stockQuote;

		public StockQuote StockQuote
		{
			get
			{
				return stockQuote;
			}
			set
			{
//				if (!string.IsNullOrEmpty(stockSymbol) && value != null 
//					&& stockSymbol.ToLower() != value.StockSymbol.ToLower())
//				{
//					/* Ignore old notification. */
//					return;
//				}
				Assign("StockQuote", ref stockQuote, value);
				/* Set the direction so that the view 
				 * doesn't need to do any evaluation. */
				if (stockQuote == null || stockQuote.ChangeInPrice == 0)
				{
					StockPriceDirection = StockPriceDirection.None;
				}
				else if (stockQuote.ChangeInPrice > 0)
				{
					StockPriceDirection = StockPriceDirection.Up;
				}
				else
				{
					StockPriceDirection = StockPriceDirection.Down;
				}
			}
		}

		StockPriceDirection stockPriceDirection;

		public StockPriceDirection StockPriceDirection
		{
			get
			{
				return stockPriceDirection;
			}
			private set
			{
				Assign("StockPriceDirection", ref stockPriceDirection, value);
			}
		}

		void subscriber_ErrorOccurred(
			object sender, NotificationChannelErrorEventArgs e)
		{
			Message = e.Message;
		}
		
		void subscriber_ChannelUriUpdated(
			object sender, NotificationChannelUriEventArgs e)
		{
			ChannelUri = e.ChannelUri;

			RegisterWithCloudService(e.ChannelUri.ToString());
		}

		public void RegisterForPushNotification()
		{
			StockQuote = null;
			Subscribe();
		}

		void RegisterWithCloudService(string pushNotificationUrl)
		{
			Message = "Registering with server.";

			StockQuoteServiceClient client = new StockQuoteServiceClient();
            //            client.TestServerCompleted += (sender, args) => Debug.WriteLine("Result received: " + args.Result);
            //            client.TestServerAsync(DateTime.Now.ToString());
			client.RegisterForStockQuoteNotificationsCompleted 
				+= delegate(object o, AsyncCompletedEventArgs args)
					{
						if (args.Error != null)
						{
							Message = "Problem registering: " + args.Error;
							return;
						}
						Message = "Registration Successful.";
						settings.StockSymbol = stockSymbol;
					};

			var subscriberId = new PushNotificationSubscriberId
			                       	{
			                       		PushNotificationUrl = pushNotificationUrl
			                       	};
			client.RegisterForStockQuoteNotificationsAsync(subscriberId, stockSymbol);
		}

		void UnregisterWithCloudService(string pushNotificationUrl)
		{
			Message = "Unregistering with server.";

			StockQuoteServiceClient client = new StockQuoteServiceClient();

			client.UnregisterForStockQuoteNotificationsCompleted 
				+= delegate(object o, AsyncCompletedEventArgs args)
						{
							if (args.Error != null)
							{
								Message = "Problem unregistering: " + args.Error;
								return;
							}
							Message = "Unregisteration Successful.";
						};

			var subscriberId = new PushNotificationSubscriberId
			                       	{
			                       		PushNotificationUrl = pushNotificationUrl
			                       	};
			client.UnregisterForStockQuoteNotificationsAsync(subscriberId);
		}

		public void UnregisterForPushNotification()
		{
			try
			{
				Message = "Unregistering for notifications.";
				if (subscriber != null)
				{
                    subscriber.Unsubscribe();
					subscriber.Dispose();
					subscriber = null;
				}
				Message = "Unregistered for notifications.";
			}
			catch (Exception ex)
			{
				Message = "Problem unregistering: " + ex.ToString();
			}
			
			if (channelUri != null)
			{
				try
				{
					UnregisterWithCloudService(channelUri.ToString());
				}
				catch (Exception ex)
				{
					Message = "Problem unregistering with server: " + ex.ToString();
				}
			}
		}

        //void SetShellTileSchedule()
        //{
        //    string url = "http://www.example.com/ImageName.png";
        //    ShellTileSchedule schedule = new ShellTileSchedule
        //    {
        //        Interval = UpdateInterval.EveryHour,
        //        MaxUpdateCount = 10,
        //        Recurrence = UpdateRecurrence.Interval,
        //        RemoteImageUri = new Uri(url),
        //        StartTime = DateTime.Now
        //    };
        //    schedule.Start();
        //}
	}
}
