﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.IO.IsolatedStorage;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;

using Microsoft.Phone.Tasks;
using Microsoft.Xna.Framework.Media;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.Sensors
{
	public class CaptureSourceViewModel: ViewModelBase
	{
		volatile bool started;

		public CaptureSourceViewModel() : base("CaptureSource")
		{
			takePhotoCommand = new DelegateCommand(obj => TakePhoto(), obj => started);

			toggleVideoCaptureCommand = new DelegateCommand(obj => ToggleCapturingVideo());

			playVideoCommand = new DelegateCommand(
				obj => PlayVideo(), 
				obj => !capturingVideo && IsVideoCaptured());
		}

		void UpdateCommands()
		{
			takePhotoCommand.RaiseCanExecuteChanged();
			playVideoCommand.RaiseCanExecuteChanged();
		}

		void TakePhoto()
		{
			if (captureSource != null && captureSource.State == CaptureState.Started)
			{
				captureSource.CaptureImageAsync();
			}
		}

		CaptureSource captureSource;

		public CaptureSource CaptureSource
		{
			get
			{
				return captureSource;
			}
		}

		public void Start()
		{
			Stop();

			if (CaptureDeviceConfiguration.AllowedDeviceAccess 
				|| CaptureDeviceConfiguration.RequestDeviceAccess())
			{
				captureSource = new CaptureSource
						{
							VideoCaptureDevice = CaptureDeviceConfiguration.GetDefaultVideoCaptureDevice(),
							AudioCaptureDevice = CaptureDeviceConfiguration.GetDefaultAudioCaptureDevice()
						};

				/* Uncomment to select desired format. */
                //captureSource.VideoCaptureDevice.DesiredFormat = captureSource.VideoCaptureDevice.SupportedFormats[1];
                
                //foreach (AudioFormat supportedFormat in captureSource.AudioCaptureDevice.SupportedFormats)
                //{
                //    if (supportedFormat.BitsPerSample > 16)
                //    {
                //        captureSource.AudioCaptureDevice.DesiredFormat = supportedFormat;
                //        break;
                //    }
                //}

                captureSource.CaptureImageCompleted += HandleCaptureImageCompleted;
				captureSource.CaptureFailed += HandleCaptureFailed;

				captureSource.Start();
				started = true;
				UpdateCommands();
			}
		}

		public void Stop()
		{
			if (captureSource != null)
			{
				captureSource.CaptureImageCompleted -= HandleCaptureImageCompleted;
				captureSource.CaptureFailed -= HandleCaptureFailed;

				if (captureSource.State == CaptureState.Started)
				{
					captureSource.Stop();
				}

                captureSource = null;
			}
			
			started = false;
			UpdateCommands();
		}

		void HandleCaptureFailed(object sender, ExceptionRoutedEventArgs e)
		{
			MessageService.ShowError(e.ErrorException.Message, "Capture Error");
		}

		void HandleCaptureImageCompleted(object sender, CaptureImageCompletedEventArgs e)
		{
			if (e.Result != null)
			{
				Deployment.Current.Dispatcher.BeginInvoke(() =>
				{
					MediaLibrary mediaLibrary = new MediaLibrary();
					string imageName = string.Format("Unleashed_{0:yyyy-MM-dd-HH-mm-ss}.jpg", DateTime.Now);
					using (MemoryStream stream = new MemoryStream())
					{
						e.Result.SaveJpeg(stream, e.Result.PixelWidth, e.Result.PixelHeight, 0, 100);
						stream.Position = 0;
						mediaLibrary.SavePicture(imageName, stream);
					}
					CapturedImages.Add(e.Result);
				});
			}
		}

		readonly DelegateCommand takePhotoCommand;

		public ICommand TakePhotoCommand
		{
			get
			{
				return takePhotoCommand;
			}
		}

		readonly ObservableCollection<WriteableBitmap> capturedImages
			= new ObservableCollection<WriteableBitmap>();

		public ObservableCollection<WriteableBitmap> CapturedImages
		{
			get
			{
				return capturedImages;
			}
		}

		readonly DelegateCommand toggleVideoCaptureCommand;

		public ICommand ToggleVideoCaptureCommand
		{
			get
			{
				return toggleVideoCaptureCommand;
			}
		}

		bool capturingVideo;

		public bool CapturingVideo
		{
			get
			{
				return capturingVideo;
			}
			private set
			{
				Assign(() => CapturingVideo, ref capturingVideo, value);
			}
		}

		readonly string videoFileName = "Unleashed_CaptureSource.mp4";

		bool IsVideoCaptured()
		{
			using (IsolatedStorageFile storageFile
					= IsolatedStorageFile.GetUserStoreForApplication())
			{
				return storageFile.FileExists(videoFileName);
			}
		}

		FileSink fileSink;

		void ToggleCapturingVideo()
		{
			if (!capturingVideo)
			{
				fileSink = new FileSink {IsolatedStorageFileName = videoFileName};

				captureSource.Stop();
				fileSink.CaptureSource = captureSource;
				captureSource.Start();
			}
			else
			{
				captureSource.Stop();
				fileSink.CaptureSource = null;
				captureSource.Start();
			}

			CapturingVideo = !capturingVideo;
			UpdateCommands();
		}

		readonly DelegateCommand playVideoCommand;

		public ICommand PlayVideoCommand
		{
			get
			{
				return playVideoCommand;
			}
		}

		void PlayVideo()
		{
			MediaPlayerLauncher mediaPlayerLauncher = new MediaPlayerLauncher
					{
						Media = new Uri(videoFileName, UriKind.Relative),
						Location = MediaLocationType.Data,
					};

			mediaPlayerLauncher.Show();
		}
	}
}
