﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;

using DanielVaughan.ComponentModel;

using Microsoft.Devices;
using Microsoft.Xna.Framework.Media;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.Sensors
{
	public class PhotoCameraViewModel: ViewModelBase
	{
		public PhotoCameraViewModel() : base("PhotoCamera")
		{
			/* Intentionally left blank. */
		}

		PhotoCamera photoCamera;

		public Camera Camera
		{
			get
			{
				return photoCamera;
			}
		}

		public void Start()
		{
			if (photoCamera != null)
			{
				return;
			}

			if (Camera.IsCameraTypeSupported(CameraType.FrontFacing))
			{
				photoCamera = new PhotoCamera(CameraType.FrontFacing);
			}
			else //if (PhotoCamera.IsCameraTypeSupported(CameraType.Primary))
			{
				photoCamera = new PhotoCamera(); /* Defaults to CameraType.Primary */
			}
			
			photoCamera.Initialized += HandlePhotoCameraInitialized;
		}

		bool cameraInitialized;

		void HandlePhotoCameraInitialized(object sender, CameraOperationCompletedEventArgs e)
		{
			cameraInitialized = true;

			PhotoCamera camera = (PhotoCamera)sender;
			/* The first element is the lowest resolution. */
			camera.Resolution = camera.AvailableResolutions.ElementAt(0);

			camera.AutoFocusCompleted += HandleAutoFocusCompleted;
			camera.CaptureImageAvailable += HandleCaptureImageAvailable;
			camera.CaptureThumbnailAvailable += HandleCaptureThumbnailAvailable;
			camera.CaptureCompleted += HandleCaptureCompleted;

			FlashModes = flashModes.Where(
							mode => photoCamera.IsFlashModeSupported(mode)).ToList();

			if (camera.IsFlashModeSupported(FlashMode.Auto))
			{
				FlashMode = FlashMode.Auto;
			}

			CameraButtons.ShutterKeyPressed += HandleShutterKeyPressed;
			CameraButtons.ShutterKeyHalfPressed += HandleShutterKeyHalfPressed;
			CameraButtons.ShutterKeyReleased += HandleShutterKeyReleased;
		}

		public void Stop()
		{
			cameraInitialized = false;

			if (photoCamera == null)
			{
				return;
			}

			photoCamera.Initialized -= HandlePhotoCameraInitialized;
			photoCamera.AutoFocusCompleted -= HandleAutoFocusCompleted;
			photoCamera.CaptureCompleted -= HandleCaptureCompleted;
			photoCamera.CaptureImageAvailable -= HandleCaptureImageAvailable;
			photoCamera.CaptureThumbnailAvailable -= HandleCaptureThumbnailAvailable;

			CameraButtons.ShutterKeyPressed -= HandleShutterKeyPressed;
			CameraButtons.ShutterKeyHalfPressed -= HandleShutterKeyHalfPressed;
			CameraButtons.ShutterKeyReleased -= HandleShutterKeyReleased;

			photoCamera.Dispose();
			photoCamera = null;
		}

		List<FlashMode> flashModes = EnumUtility.CreateEnumValueList<FlashMode>().ToList();

		public IEnumerable<FlashMode> FlashModes
		{
			get
			{
				return flashModes;
			}
			private set
			{
				Assign(() => FlashModes, ref flashModes, (List<FlashMode>)value);
			}
		}

		FlashMode flashMode = FlashMode.Auto;

		public FlashMode FlashMode
		{
			get
			{
				return flashMode;
			}
			set
			{
				if (Assign("FlashMode", ref flashMode, value) == AssignmentResult.Success && cameraInitialized)
				{
					if (photoCamera.IsFlashModeSupported(value))
					{
						photoCamera.FlashMode = value;
					}
				}
			}
		}

		void HandleShutterKeyHalfPressed(object sender, EventArgs e)
		{
			VisualState = VisualStateValue.Focusing;
			photoCamera.Focus();
		}

		volatile string fileName;

		void HandleShutterKeyPressed(object sender, EventArgs e)
		{
			fileName = string.Format(
							"Unleashed_{0:yyyy-MM-dd-HH-mm-ss}.jpg", DateTime.Now);
			photoCamera.CaptureImage();
		}

		void HandleShutterKeyReleased(object sender, EventArgs e)
		{
			photoCamera.CancelFocus();

			VisualState = VisualStateValue.NotFocusing;
		}

		void HandleCaptureImageAvailable(object sender, ContentReadyEventArgs e)
		{
			Deployment.Current.Dispatcher.BeginInvoke(
				() => SaveImage(e.ImageStream,
									(int)photoCamera.Resolution.Width,
									(int)photoCamera.Resolution.Height));
		}
		
		void SaveImage(Stream imageStream, int width, int height)
		{
			ArgumentValidator.AssertNotNull(imageStream, "imageStream");

			var mediaLibrary = new MediaLibrary();
			mediaLibrary.SavePicture(fileName, imageStream);
			imageStream.Position = 0;

			var bitmap = new WriteableBitmap(width, height);
			bitmap.LoadJpeg(imageStream);

			CapturedImages.Add(bitmap);
		}

		static string imageDirectory = "PhotoCameraImages";

		public static string ImageDirectory
		{
			get
			{
				return imageDirectory;
			}
		}

		void HandleCaptureThumbnailAvailable(object sender, ContentReadyEventArgs e)
		{
			using (e.ImageStream)
			{
				/* Save thumbnail as JPEG to isolated storage. */
				using (IsolatedStorageFile storageFile
							= IsolatedStorageFile.GetUserStoreForApplication())
				{
					if (!storageFile.DirectoryExists(imageDirectory))
					{
						storageFile.CreateDirectory(imageDirectory);
					}

					using (IsolatedStorageFileStream targetStream
								= storageFile.OpenFile(
									imageDirectory + "/" + fileName, 
									FileMode.Create, FileAccess.Write))
					{
						/* Initialize the buffer for 4KB disk pages. */
						byte[] readBuffer = new byte[4096];
						int bytesRead;

						/* Copy the thumbnail to isolated storage. */
						while ((bytesRead = e.ImageStream.Read(
												readBuffer, 0, readBuffer.Length)) > 0)
						{
							targetStream.Write(readBuffer, 0, bytesRead);
						}
					}
				}
			}
		}

		void HandleCaptureCompleted(
			object sender, CameraOperationCompletedEventArgs e)
		{
			VisualState = VisualStateValue.NotFocusing;
		}

		void HandleAutoFocusCompleted(
			object sender, CameraOperationCompletedEventArgs e)
		{
			VisualState = VisualStateValue.Focusing;
		}
	
		readonly ObservableCollection<WriteableBitmap> capturedImages 
					= new ObservableCollection<WriteableBitmap>();

		public ObservableCollection<WriteableBitmap> CapturedImages
		{
			get
			{
				return capturedImages;
			}
		}

		public void InitializeEffectImage(int width, int height)
		{
			EffectImage = new WriteableBitmap(width, height);
			Thread previewImageProcessingThread = new Thread(ProcessEffectFrames);
			previewImageProcessingThread.Start();
		}

		WriteableBitmap effectImage;

		public ImageSource EffectImage
		{
			get
			{
				return effectImage;
			}
			private set
			{
				Assign(() => EffectImage, ref effectImage, (WriteableBitmap)value);
			}
		}

		volatile bool effectEnabled;

		public bool EffectEnabled
		{
			get
			{
				return effectEnabled;
			}
			set
			{
				if (Assign(() => EffectEnabled, ref effectEnabled, value) 
														== AssignmentResult.Success)
				{
					SetEffectState(value);
				}
			}
		}

		Thread effectThread;

		void SetEffectState(bool enabled)
		{
			if (enabled)
			{
				EffectImage = new WriteableBitmap(
					(int)photoCamera.PreviewResolution.Width, 
					(int)photoCamera.PreviewResolution.Height);

				effectThread = new Thread(ProcessEffectFrames);
				effectThread.Start();
			}

			VisualState = enabled ? VisualStateValue.EffectOn
									: VisualStateValue.EffectOff;
		}

		readonly AutoResetEvent frameResetEvent = new AutoResetEvent(true);

		void ProcessEffectFrames()
		{
			try
			{
				while (effectEnabled)
				{
					frameResetEvent.WaitOne();

					int width = (int)photoCamera.PreviewResolution.Width;
					int height = (int)photoCamera.PreviewResolution.Height;
					int[] pixelArray = new int[width * height];

					photoCamera.GetPreviewBufferArgb32(pixelArray);
	
					int[] effectPixels = new int[pixelArray.Length];

					for (int i = 0; i < pixelArray.Length; i++)
					{
						effectPixels[i] = ConvertColorToGrayScale(pixelArray[i]);
					}
					
					Deployment.Current.Dispatcher.BeginInvoke(
						delegate
							{
								/* Copy to the WriteableBitmap. */
								effectPixels.CopyTo(effectImage.Pixels, 0);

								effectImage.Invalidate();
								frameResetEvent.Set();
							});
				}
			}
			catch (Exception ex)
			{
				Debug.WriteLine("Unable to process effect frames." + ex);
				EffectEnabled = false;
				frameResetEvent.Set();
			}
		}

		#region Effect processing

		int ConvertColorToGrayScale(int color)
		{
			int alpha, red, green, blue;
			ConvertColorToArgb(color, out alpha, out red, out green, out blue);

			int average = (red + green + blue) / 3;
			red = green = blue = average;

			int result = ConvertArgbToColor(alpha, red, green, blue);
			return result;
		}

		void ConvertColorToArgb(
				int color, out int alpha, out int red, out int green, out int blue)
		{
			alpha = color >> 24;
			red = (color & 0x00ff0000) >> 16;
			green = (color & 0x0000ff00) >> 8;
			blue = (color & 0x000000ff);
		}

		int ConvertArgbToColor(int alpha, int red, int green, int blue)
		{
			int result = ((alpha & 0xFF) << 24) 
							| ((red & 0xFF) << 16) 
							| ((green & 0xFF) << 8) 
							| (blue & 0xFF);
			return result;
		}

		#endregion

		public enum VisualStateValue
		{
			EffectOff,
			EffectOn,
			NotFocusing,
			Focusing
		}

		VisualStateValue visualState;

		public VisualStateValue VisualState
		{
			get
			{
				return visualState;
			}
			private set
			{
				Assign(() => VisualState, ref visualState, value);
			}
		}
	}
}