﻿
using System;
using System.Diagnostics;
using System.Windows.Input;

using DanielVaughan.Devices.Sensors;
using DanielVaughan.WindowsPhone7Unleashed.Devices.Sensors;

using Microsoft.Devices.Sensors;
using Microsoft.Xna.Framework;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.Sensors
{
	public enum CompassOrientation
	{
		Flat,
		Portrait
	}

	public class CompassViewModel: ViewModelBase
	{
		Compass compass;
		readonly ReadingSmoother readingSmoother = new ReadingSmoother(new LowPassFilterStrategy());

		Accelerometer accelerometer;

		public CompassViewModel() : base("compass")
		{
			toggleCalibrationCommand = new DelegateCommand(obj => ToggleCalibration());
		}

		void ToggleCalibration()
		{
			VisualState = VisualState == VisualStateValue.Calibrating
			              	? VisualStateValue.NotCalibrating
			              	: VisualStateValue.Calibrating;
		}

		public void Start()
		{
			if (!Compass.IsSupported)
			{
				MessageService.ShowMessage("Compass is not supported on this device.");
				return;
			}

			if (compass != null)
			{
				return;
			}

			compass = new Compass();
			compass.TimeBetweenUpdates = TimeSpan.FromMilliseconds(updateIntervalMs);
			UpdateIntervalMs = (int)compass.TimeBetweenUpdates.TotalMilliseconds;

			compass.CurrentValueChanged += HandleCompassCurrentValueChanged;
			compass.Calibrate += HandleCompassCalibrate;
			compass.Start();

			accelerometer = new Accelerometer();
			accelerometer.CurrentValueChanged += HandleAccelerometerCurrentValueChanged;
			accelerometer.Start();
		}

		public void Stop()
		{
			if (compass != null)
			{
				compass.CurrentValueChanged -= HandleCompassCurrentValueChanged;
				compass.Calibrate -= HandleCompassCalibrate;

				compass.Stop();
				compass.Dispose();
				compass = null;
			}

			if (accelerometer != null)
			{
				accelerometer.CurrentValueChanged -= HandleAccelerometerCurrentValueChanged;
				accelerometer.Stop();
				accelerometer.Dispose();
				accelerometer = null;
			}
		}

		double smoothedHeading;

		public double SmoothedHeading
		{
			get
			{
				return smoothedHeading;
			}
			private set
			{
				Assign("SmoothedHeading", ref smoothedHeading, value);
			}
		}

		void HandleCompassCurrentValueChanged(object sender, SensorReadingEventArgs<CompassReading> e)
		{
			MagneticHeading = e.SensorReading.MagneticHeading;
			TrueHeading = e.SensorReading.TrueHeading;
			HeadingAccuracy = e.SensorReading.HeadingAccuracy;
			Vector3 reading = e.SensorReading.MagnetometerReading;
			MagnetometerReading = new ThreeDimensionalVector(reading.X,
			                                                 reading.Y,
			                                                 reading.Z);

			SmoothedHeading = readingSmoother.ProcessReading(e.SensorReading.MagneticHeading);
		}

		void HandleCompassCalibrate(object sender, CalibrationEventArgs e)
		{
			VisualState = VisualStateValue.Calibrating;
		}

		double magneticHeading;

		public double MagneticHeading
		{
			get
			{
				return magneticHeading;
			}
			private set
			{
				Assign("MagneticHeading", ref magneticHeading, value);
			}
		}

		double trueHeading;

		public double TrueHeading
		{
			get
			{
				return trueHeading;
			}
			private set
			{
				Assign("TrueHeading", ref trueHeading, value);
			}
		}

		double headingAccuracy;

		public double HeadingAccuracy
		{
			get
			{
				return headingAccuracy;
			}
			private set
			{
				Assign("HeadingAccuracy", ref headingAccuracy, value);
			}
		}

		ThreeDimensionalVector magnetometerReading;

		public ThreeDimensionalVector MagnetometerReading
		{
			get
			{
				return magnetometerReading;
			}
			private set
			{
				Assign("MagnetometerReading", ref magnetometerReading, value);
			}
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			private set
			{
				Assign(() => Message, ref message, value);
			}
		}

		int updateIntervalMs;

		public int UpdateIntervalMs
		{
			get
			{
				return updateIntervalMs;
			}
			private set
			{
				Assign(() => UpdateIntervalMs, ref updateIntervalMs, value);
			}
		}

		#region Compass Orientation
		void HandleAccelerometerCurrentValueChanged(object sender, SensorReadingEventArgs<AccelerometerReading> e)
		{
			Vector3 accelerationVector = e.SensorReading.Acceleration;

			bool usingNegativeZAxis = false;

			/* Determine the orientation of the device. */
			if (Math.Abs(accelerationVector.Z) < Math.Cos(Math.PI / 4)
				&& accelerationVector.Y < Math.Sin(7 * Math.PI / 4))
			{
				usingNegativeZAxis = true;
			}

			CompassOrientation = usingNegativeZAxis ? CompassOrientation.Portrait : CompassOrientation.Flat;
		}

		CompassOrientation compassOrientation;

		public CompassOrientation CompassOrientation
		{
			get
			{
				return compassOrientation;
			}
			private set
			{
				Assign(() => CompassOrientation, ref compassOrientation, value);
			}
		}
		#endregion

		readonly DelegateCommand toggleCalibrationCommand;

		public ICommand ToggleCalibrationCommand
		{
			get
			{
				return toggleCalibrationCommand;
			}
		}

		public enum VisualStateValue
		{
			NotCalibrating,
			Calibrating
		}

		VisualStateValue visualState;

		public VisualStateValue VisualState
		{
			get
			{
				return visualState;
			}
			private set
			{
				Assign(() => VisualState, ref visualState, value);
			}
		}
	}
}
