﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Windows;
using System.Windows.Controls;

using Microsoft.Devices.Radio;
using Microsoft.Phone.Controls.Primitives;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.Sensors
{
	public class FMRadioViewModel : ViewModelBase
	{
		Timer timer;
		static readonly int timerIntervalMs = 1000;

		public FMRadioViewModel() : base("fm radio")
		{
			timer = new Timer(HandleTimerTick, null, timerIntervalMs, timerIntervalMs);
			UpdateFrequencyRanges();

			frequenciesFractional.SelectedItem = (int)(Frequency % 1 * 10);
			frequenciesFractional.SelectionChanged += HandleFrequencyFractionPartChanged;
		}

		public double Frequency
		{
			get
			{
				return FMRadio.Instance.Frequency < 1
					? frequencyMin : FMRadio.Instance.Frequency;
			}
			set
			{
				if (Math.Abs(FMRadio.Instance.Frequency - value) >= .1)
				{
					FMRadio.Instance.Frequency = value;
					OnPropertyChanged(() => Frequency);
				}
			}
		}

		readonly IEnumerable<RadioRegion> radioRegions 
						= EnumUtility.CreateEnumValueList<RadioRegion>();

		public IEnumerable<RadioRegion> Regions
		{
			get
			{
				return radioRegions;
			}
		}

		public RadioRegion Region
		{
			get
			{
				return FMRadio.Instance.CurrentRegion;
			}
			set
			{
				FMRadio fmRadio = FMRadio.Instance;
				if (fmRadio.CurrentRegion != value)
				{
					fmRadio.CurrentRegion = value;
					UpdateFrequencyRanges();

					OnPropertyChanged(() => Region);
				}
			}
		}

		public bool PoweredOn
		{
			get
			{
				return FMRadio.Instance.PowerMode == RadioPowerMode.On;
			}
			set
			{
				bool state = FMRadio.Instance.PowerMode == RadioPowerMode.On;
				if (state != value)
				{
					FMRadio.Instance.PowerMode = value 
						? RadioPowerMode.On : RadioPowerMode.Off;

					if (value)
					{
						ThreadPool.QueueUserWorkItem(
							delegate
								{
									Wait(2000);
									if (SignalStrength < 0.01)
									{
										MessageService.ShowMessage(
											"This phone uses your headphones " 
											+ "as an FM radio antenna. " 
											+ "To listen to radio, connect your headphones.", 
											"No antenna");
									}
								});
					}

					OnPropertyChanged(() => PoweredOn);

					if (FMRadio.Instance.PowerMode == RadioPowerMode.On)
					{
						OnPropertyChanged(() => Frequency);
					}
				}
			}
		}

		public double SignalStrength
		{
			get
			{
				return FMRadio.Instance.SignalStrength;
			}
		}

		double frequencyMin;

		public double FrequencyMin
		{
			get
			{
				return frequencyMin;
			}
			private set
			{
				Assign(() => FrequencyMin, ref frequencyMin, value);
			}
		}

		double frequencyMax;

		public double FrequencyMax
		{
			get
			{
				return frequencyMax;
			}
			private set
			{
				Assign(() => FrequencyMax, ref frequencyMax, value);
			}
		}

		void UpdateFrequencyRanges()
		{
			if (frequenciesNonFractional != null)
			{
				frequenciesNonFractional.SelectionChanged -= HandleFrequencyLargeChanged;
			}

			NumericDataSource dataSource = new NumericDataSource();
			
			switch (Region)
			{
				case RadioRegion.Europe:
					FrequencyMin = 88;
					FrequencyMax = 108;
					dataSource.Minimum = 88;
					dataSource.Maximum = 108;
					break;
				case RadioRegion.Japan:
					FrequencyMin = 76;
					FrequencyMax = 90;
					dataSource.Minimum = 76;
					dataSource.Maximum = 90;
					break;
				case RadioRegion.UnitedStates:
					FrequencyMin = 88.1;
					FrequencyMax = 107.9;
					dataSource.Minimum = 88;
					dataSource.Maximum = 107;
					break;
			}

			double tempFrequency = Frequency;
			dataSource.SelectedNumber = (int)(tempFrequency - tempFrequency % 1);
			FrequenciesNonFractional = dataSource;
			dataSource.SelectionChanged += HandleFrequencyLargeChanged;
		}

		void Wait(int delayMs)
		{
			object waitLock = new object();

			if (delayMs > 0)
			{
				lock (waitLock)
				{
					Monitor.Wait(waitLock, delayMs);
				}
			}
		}

		NumericDataSource frequenciesNonFractional = new NumericDataSource();

		public ILoopingSelectorDataSource FrequenciesNonFractional
		{
			get
			{
				return frequenciesNonFractional;
			}
			private set
			{
				Assign(() => FrequenciesNonFractional,
								ref frequenciesNonFractional, (NumericDataSource)value);
			}
		}

		readonly NumericDataSource frequenciesFractional
					= new NumericDataSource { Minimum = 0, Maximum = 9 };

		public ILoopingSelectorDataSource FrequenciesFractional
		{
			get
			{
				return frequenciesFractional;
			}
		}

		void HandleFrequencyFractionPartChanged(object sender, SelectionChangedEventArgs e)
		{
			NumericDataSource dataSource = (NumericDataSource)sender;
			double f = Frequency;
			double newFrequency = f - f % 1 + dataSource.SelectedNumber / 10.0;
			if (newFrequency > frequencyMax || newFrequency < frequencyMin)
			{
				return;
			}
			Frequency = newFrequency;
		}

		void HandleFrequencyLargeChanged(object sender, SelectionChangedEventArgs e)
		{
			NumericDataSource dataSource = (NumericDataSource)sender;
			var newFrequency = Frequency % 1 + dataSource.SelectedNumber;
			if (newFrequency > frequencyMax || newFrequency < frequencyMin)
			{
				return;
			}
			Frequency = newFrequency;
		}

		double previousFrequency;

		void HandleTimerTick(object state)
		{
			OnPropertyChanged(() => SignalStrength);
			double frequency = FMRadio.Instance.Frequency;
			if (Math.Abs(previousFrequency - frequency) >= 0.1)
			{
				double fraction = frequency % 1;
				Deployment.Current.Dispatcher.BeginInvoke(delegate
				{
					frequenciesFractional.SelectedNumber = (int)(fraction * 10);
					frequenciesNonFractional.SelectedNumber = (int)(frequency - fraction);
					OnPropertyChanged(() => Frequency);
				});
			}
			previousFrequency = frequency;
		}
	}
}
