﻿using System;
using System.Diagnostics;
using System.IO;
using System.IO.IsolatedStorage;
using System.Windows.Input;

using Microsoft.Xna.Framework.Audio;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.Sensors
{
	public class MicrophoneViewModel : ViewModelBase
	{
		readonly Microphone microphone = Microphone.Default;
		readonly byte[] buffer;
		const string fileName = "MicrophoneSample.bytes";
		MemoryStream stream = new MemoryStream();
		readonly object fileLock = new object();

		public MicrophoneViewModel() : base("microphone")
		{
			microphone.BufferDuration = TimeSpan.FromSeconds(1);
			microphone.BufferReady += HandleBufferReady;

			int sampleSizeInBytes = microphone.GetSampleSizeInBytes(microphone.BufferDuration);
			buffer = new byte[sampleSizeInBytes];

			toggleRecordCommand = new DelegateCommand(obj => ToggleRecording());
			playCommand = new DelegateCommand(obj => Play(), obj => !recording);
		}
		
		void UpdateCommands()
		{
			playCommand.RaiseCanExecuteChanged();
		}

		void StartRecording()
		{
			stream = new MemoryStream();
			microphone.Start();
			Recording = true;
			UpdateCommands();
		}

		void HandleBufferReady(object sender, EventArgs e)
		{
			//			long usedBytes = Microsoft.Phone.Info.DeviceStatus.ApplicationCurrentMemoryUsage;
			//			long limitBytes = Microsoft.Phone.Info.DeviceStatus.ApplicationMemoryUsageLimit;
			//			if ((usedBytes + buffer.Length + 1048576) >= limitBytes) // 1 048 576 = 1 MB
			//			{
			//				/* Rather than giving up, we could write 
			//				 * the stream to isolated storage and continue on. */
			//				MessageService.ShowWarning("Recording has exceeded phone memory.");
			//				StopRecording();
			//				return;
			//			}
			//			Debug.WriteLine("Memory used:{0} remaining: {1}", usedBytes, limitBytes - usedBytes);

			microphone.GetData(buffer);
			stream.Write(buffer, 0, buffer.Length);

			if (!recording)
			{
				stream.Flush();
				microphone.Stop();
				WriteFile(stream);
			}
		}

		void StopRecording()
		{
			if (microphone.State != MicrophoneState.Stopped)
			{
				/* Stopping the Microphone here prevents the last BufferReady event 
				 * from being raised, which can cause the audio to be cut off. 
				 * A request to stop recording is therefore dealt with 
				 * in the HandleBufferReady method. */
				Recording = false;
				UpdateCommands();
			}
		}

		void ToggleRecording()
		{
			if (recording)
			{
				StopRecording();
			}
			else
			{
				StartRecording();
			}
		}

		void WriteFile(MemoryStream memoryStream)
		{
			lock (fileLock)
			{
				using (var isolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication())
				{
					using (IsolatedStorageFileStream fileStream
						= isolatedStorageFile.OpenFile(fileName, FileMode.Create))
					{
						memoryStream.WriteTo(fileStream);
					}
				}
			}
		}

		void Play()
		{
			byte[] bytes = null;

			lock (fileLock)
			{
				using (var userStore = IsolatedStorageFile.GetUserStoreForApplication())
				{
					if (userStore.FileExists(fileName))
					{
						using (IsolatedStorageFileStream fileStream
									= userStore.OpenFile(fileName, FileMode.Open, FileAccess.Read))
						{
							bytes = new byte[fileStream.Length];
							fileStream.Read(bytes, 0, bytes.Length);
						}
					}
				}
			}

			if (bytes != null)
			{
				PlayAudio(bytes);
			}
		}

		double pitch;

		public double Pitch
		{
			get
			{
				return pitch;
			}
			set
			{
				Assign(() => Pitch, ref pitch, value);
				if (effectInstance != null)
				{
					effectInstance.Pitch = (float)value;
				}
			}
		}

		SoundEffectInstance effectInstance;

		void PlayAudio(byte[] audioBytes)
		{
			if (audioBytes == null || audioBytes.Length == 0 
				|| effectInstance != null 
				&& effectInstance.State == SoundState.Playing)
			{
				return;
			}

			var soundEffect = new SoundEffect(audioBytes, microphone.SampleRate, AudioChannels.Mono);
			if (effectInstance != null)
			{
				effectInstance.Dispose();
			}
			effectInstance = soundEffect.CreateInstance();
			effectInstance.Pitch = (float)pitch;
			effectInstance.Play();
		}

		readonly DelegateCommand playCommand;

		public ICommand PlayCommand
		{
			get
			{
				return playCommand;
			}
		}
		
		readonly DelegateCommand toggleRecordCommand;

		public ICommand ToggleRecordCommand
		{
			get
			{
				return toggleRecordCommand;
			}
		}

		bool recording;

		public bool Recording
		{
			get
			{
				return recording;
			}
			private set
			{
				Assign(() => Recording, ref recording, value);
			}
		}
	}
}
