﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.Serialization.Json;
using System.Text;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class PushNotificationSendResult
	{
		public NotificationStatus NotificationStatus { get; private set; }
		public DeviceConnectionStatus DeviceConnectionStatus { get; private set; }
		public SubscriptionStatus SubscriptionStatus { get; private set; }

		public PushNotificationSendResult(
			NotificationStatus notificationStatus, 
			DeviceConnectionStatus deviceConnectionStatus, 
			SubscriptionStatus subscriptionStatus)
		{
			NotificationStatus = notificationStatus;
			DeviceConnectionStatus = deviceConnectionStatus;
			SubscriptionStatus = subscriptionStatus;
		}

		internal PushNotificationSendResult(
			string notificationStatus, string deviceConnectionStatus, 
			string subscriptionStatus) : this(
			PushNotificationEnumConverter.ToNotificationStatus(notificationStatus),
			PushNotificationEnumConverter.ToDeviceConnectionStatus(
				deviceConnectionStatus),
			PushNotificationEnumConverter.ToSubscriptionStatus(subscriptionStatus))
		{
			/* Intentionally left blank. */
		}

		public override string ToString()
		{
			return string.Format("NotificationStatus={0}, DeviceConnectionStatus={1}, " 
					+ "SubscriptionStatus={2}", 
				NotificationStatus, DeviceConnectionStatus, SubscriptionStatus);
		}
	}

	public class PushNotifier
	{
		public PushNotificationSendResult SendToastNotification(
			string title, string message, string url, PushNotificationPriority priority)
		{
			/* You can choose from one of these three batching intervals 
			 * for toast notifications:
			 * 2 – Real Time. The toast notification is delivered as soon as possible.
			 * 12 – Priority. The toast notification is delivered at a predefined timeout.
			 * 22 – Regular. The toast notification is delivered at a predefined timeout 
			 *		which is greater than the Priority batching interval. */
			string priorityHeader;
			if (priority == PushNotificationPriority.RealTime)
			{
				priorityHeader = "2";
			}
			else if (priority == PushNotificationPriority.Priority)
			{
				priorityHeader = "12";
			}
			else
			{
				priorityHeader = "22";
			}

			/* Create the http message that will be sent 
			 * to the Microsoft hosted server. */
			HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);
			request.Method = "POST";
			request.ContentType = "text/xml";
			request.Headers = new WebHeaderCollection
			                  	{
									{ "X-WindowsPhone-Target", "toast" },
			                  		{ "X-NotificationClass", priorityHeader },
									{ "X-MessageID", Guid.NewGuid().ToString() }
			                  	};

			/* The XML envelope contains the notification text. */
			string messageFormat = "<?xml version='1.0' encoding='utf-8'?>" +
								"<wp:Notification xmlns:wp='WPNotification'>" +
									"<wp:Toast>" +
										"<wp:Text1>{0}</wp:Text1>" +
										"<wp:Text2>{1}</wp:Text2>" +
									"</wp:Toast>" +
								"</wp:Notification>";

			/* Insert the message string. */
			string messageFormatted = string.Format(messageFormat, title, message);
			/* The message will be sent as a byte array. */
			byte[] messageBytes = Encoding.Default.GetBytes(messageFormatted);

			request.ContentLength = messageBytes.Length;

			using (Stream requestStream = request.GetRequestStream())
			{
				requestStream.Write(messageBytes, 0, messageBytes.Length);
			}

			/* Get the response after the message is sent. */
			HttpWebResponse response = (HttpWebResponse)request.GetResponse();
			string notificationStatus = response.Headers["X-NotificationStatus"];
			string deviceConnectionStatus = response.Headers["X-DeviceConnectionStatus"];
			string subscriptionStatus = response.Headers["X-SubscriptionStatus"];
			string messageId = response.Headers["X-MessageID"];

			PushNotificationSendResult result = new PushNotificationSendResult(
				notificationStatus, deviceConnectionStatus, subscriptionStatus);

			Debug.WriteLine(result);
			return result;
		}

		public PushNotificationSendResult SendTileNotification(string title, int count, 
			string imagePath, string url, PushNotificationPriority priority)
		{
			/* You can choose from one of these three batching intervals 
			 * for tile notifications: 
			 * 1 – Real Time. The tile notification is delivered as soon as possible.
			 * 11 – Priority. The tile notification is delivered at a predefined timeout.
			 * 21 – Regular. The tile notification is delivered at a predefined timeout, 
			 * which is greater than the Priority batching interval. */
			string priorityHeader;
			if (priority == PushNotificationPriority.RealTime)
			{
				priorityHeader = "1";
			}
			else if (priority == PushNotificationPriority.Priority)
			{
				priorityHeader = "11";
			}
			else
			{
				priorityHeader = "21";
			}

			/* Create the http message that will be sent to the Microsoft hosted server. */
			HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);
			request.Method = "POST";
			request.ContentType = "text/xml";
			request.Headers = new WebHeaderCollection
			                  	{
									{ "X-WindowsPhone-Target", "token" },
			                  		{ "X-NotificationClass", priorityHeader }
			                  	};

			/* The XML envelope contains the notification text. */
			string messageFormat = "<?xml version='1.0' encoding='utf-8'?>" +
								"<wp:Notification xmlns:wp='WPNotification'>" +
									"<wp:Tile>" +
										"<wp:BackgroundImage>{0}</wp:BackgroundImage>" +
										"<wp:Count>{1}</wp:Count>" +
										"<wp:Title>{2}</wp:Title>" +
									"</wp:Tile>" +
								"</wp:Notification>";

			/* Insert the message string. */
			string messageFormatted = string.Format(
				messageFormat, imagePath, count, title);
			/* The message will be sent as a byte array. */
			byte[] messageBytes = Encoding.Default.GetBytes(messageFormatted);

			request.ContentLength = messageBytes.Length;

			using (Stream requestStream = request.GetRequestStream())
			{
				requestStream.Write(messageBytes, 0, messageBytes.Length);
			}

			/* Get the response after the message is sent. */
			HttpWebResponse response = (HttpWebResponse)request.GetResponse();
			string notificationStatus = response.Headers["X-NotificationStatus"];
			string subscriptionStatus = response.Headers["X-SubscriptionStatus"];
			string deviceConnectionStatus = response.Headers["X-DeviceConnectionStatus"];

			PushNotificationSendResult result = new PushNotificationSendResult(
				PushNotificationEnumConverter.ToNotificationStatus(notificationStatus),
				PushNotificationEnumConverter.ToDeviceConnectionStatus(deviceConnectionStatus),
				PushNotificationEnumConverter.ToSubscriptionStatus(subscriptionStatus)
				);
			Debug.WriteLine(result);
			return result;
		}

		public PushNotificationSendResult SendRawNotification(
			object content, string url, PushNotificationPriority priority)
		{
			/* You can choose from one of these three batching intervals 
			 * for raw notifications:
			 * 3-10 –	Real Time. The raw notification is delivered 
			 *			as soon as possible.
			 * 13-20 –	Priority. The raw notification is delivered 
			 *			at a predefined timeout.
			 * 23-31 –	Regular. The raw notification is delivered 
			 *			at a predefined timeout which is greater 
			 *			than the Priority batching interval. */
			string priorityHeader;
			if (priority == PushNotificationPriority.RealTime)
			{
				priorityHeader = "5";
			}
			else if (priority == PushNotificationPriority.Priority)
			{
				priorityHeader = "16";
			}
			else
			{
				priorityHeader = "26";
			}

			/* Create the http message that will be sent 
			 * to the Microsoft hosted server. */
			HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);
			/* HTTP POST is the only allowed method to send the notification. */
			request.Method = "POST";
			request.ContentType = "application/json; charset=utf-8";
			request.Headers = new WebHeaderCollection {
				{ "X-MessageID", Guid.NewGuid().ToString() }, 
				{ "X-NotificationClass", priorityHeader } };

			/* The message will be sent as a byte array. */
			byte[] contentBytes;
			var serializer = new DataContractJsonSerializer(content.GetType());

			using (MemoryStream stream = new MemoryStream())
			{
				serializer.WriteObject(stream, content);
				contentBytes = stream.ToArray();
			}

			request.ContentLength = contentBytes.Length;
			
			using (Stream requestStream = request.GetRequestStream())
			{
				requestStream.Write(contentBytes, 0, contentBytes.Length);
			}

			/* Get the response after the message is sent. */
			HttpWebResponse response = (HttpWebResponse)request.GetResponse();
			string notificationStatus = response.Headers["X-NotificationStatus"];
			string subscriptionStatus = response.Headers["X-SubscriptionStatus"];
			string deviceConnectionStatus = response.Headers["X-DeviceConnectionStatus"];
			string messageId = response.Headers["X-MessageID"];

			PushNotificationSendResult result = CreateSendResult(
				notificationStatus, deviceConnectionStatus, subscriptionStatus);

			return result;
		}

		PushNotificationSendResult CreateSendResult(
			string notificationStatus, string deviceConnectionStatus, 
			string subscriptionStatus)
		{
			PushNotificationSendResult result = new PushNotificationSendResult(
				PushNotificationEnumConverter.ToNotificationStatus(notificationStatus),
				PushNotificationEnumConverter.ToDeviceConnectionStatus(
					deviceConnectionStatus),
				PushNotificationEnumConverter.ToSubscriptionStatus(subscriptionStatus)
			);
			Debug.WriteLine(result);
			return result;
		}

	}
}