﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Navigation;

using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input.Touch;

namespace SilverlightXnaHybrid
{
	public partial class GamePage : PhoneApplicationPage
	{
		/* XNA Setup */
		readonly GameTimer gameTimer = new GameTimer();
		SpriteBatch spriteBatch;

		/* Required for Silverlight UI rendering. */
		UIElementRenderer uiElementRenderer;

		/* Scene background. */
		Texture2D background;

		/* XNA touch setup. */
		float cameraFieldOfView = 45; // Initial camera field of view (serves as a zoom level)
		float rotationX;
		float rotationY;
		float? previousGestureValue;

		readonly XnaModelWrapper modelWrapper 
			= new XnaModelWrapper { Lights = new[] { true, true, true } };

		readonly GamePageViewModel viewModel;

		ModelMetadata GetModelMetadata()
		{
			return viewModel.ModelMetadata;
		}

		public GamePage()
		{
			InitializeComponent();
			DataContext = viewModel = new GamePageViewModel();

			gameTimer.Update += HandleGameTimerUpdate;
			gameTimer.Draw += HandleGameTimerDraw;

			LayoutUpdated += HandleLayoutUpdated;

			/* Enable gestures - Pinch for zoom and horizontal drag for rotate. */
			TouchPanel.EnabledGestures 
				= GestureType.FreeDrag 
					| GestureType.Pinch 
					| GestureType.PinchComplete;
		}

		protected override void OnNavigatedTo(NavigationEventArgs e)
		{
			GraphicsDevice graphicsDevice
				= SharedGraphicsDeviceManager.Current.GraphicsDevice;
			/* Set the sharing mode of the graphics device 
			 * to turn on XNA rendering. */
			graphicsDevice.SetSharingMode(true);

			spriteBatch = new SpriteBatch(graphicsDevice);

			App app = (App)Application.Current;
			ContentManager contentManager = app.ContentManager;

			background = contentManager.Load<Texture2D>("Background");

			string modelName = GetModelMetadata().Asset;
			Model xnaModel = contentManager.Load<Model>(modelName);
			modelWrapper.Load(xnaModel);

			gameTimer.Start();

			base.OnNavigatedTo(e);
		}

		protected override void OnNavigatedFrom(NavigationEventArgs e)
		{
			gameTimer.Stop();

			/* Set the sharing mode of the graphics device to turn off XNA rendering. */
			SharedGraphicsDeviceManager.Current.GraphicsDevice.SetSharingMode(false);

			base.OnNavigatedFrom(e);
		}

		void HandleLayoutUpdated(object sender, EventArgs e)
		{
			if (uiElementRenderer == null
				|| LayoutRoot.ActualWidth > 0 && LayoutRoot.ActualHeight > 0)
			{
				uiElementRenderer = new UIElementRenderer(
					LayoutRoot,
					(int)LayoutRoot.ActualWidth,
					(int)LayoutRoot.ActualHeight);
			}
		}

		void HandleGameTimerUpdate(object sender, GameTimerEventArgs e)
		{
			ProcessInput();

			float yaw = MathHelper.Pi + MathHelper.PiOver2 + rotationX / 100;
			float pitch = rotationY / 100;
			ModelMetadata metadata = GetModelMetadata();
			modelWrapper.Rotation
				= metadata.World * Matrix.CreateFromYawPitchRoll(yaw, pitch, 0);
			modelWrapper.View = metadata.ViewMatrix;
			modelWrapper.IsTextureEnabled = true;
			modelWrapper.IsPerPixelLightingEnabled = true;
			modelWrapper.Projection = Matrix.CreatePerspectiveFieldOfView(
				MathHelper.ToRadians(cameraFieldOfView) / metadata.FieldOfViewDivisor,
				metadata.AspectRatio,
				metadata.NearPlaneDistance,
				metadata.FarPlaneDistance);
		}

		void HandleGameTimerDraw(object sender, GameTimerEventArgs e)
		{
			GraphicsDevice graphicsDevice 
				= SharedGraphicsDeviceManager.Current.GraphicsDevice;

			graphicsDevice.Clear(Color.CornflowerBlue);

			/* Draw the background. */
			spriteBatch.Begin();
			spriteBatch.Draw(background, Vector2.Zero, Color.White);
			spriteBatch.End();
			
			graphicsDevice.DepthStencilState = DepthStencilState.Default;
			graphicsDevice.BlendState = BlendState.Opaque;
			graphicsDevice.RasterizerState = RasterizerState.CullCounterClockwise;
			graphicsDevice.SamplerStates[0] = SamplerState.LinearWrap;

			/* Draw the XNA model. */
			modelWrapper.Draw();

			/* Render the Silverlight UI. */
			uiElementRenderer.Render();

			/* Draw the Silverlight elements. */
			spriteBatch.Begin();
			spriteBatch.Draw(uiElementRenderer.Texture, Vector2.Zero, Color.White);
			spriteBatch.End();
		}

		void ProcessInput()
		{
			while (TouchPanel.IsGestureAvailable)
			{
				GestureSample gestureSample = TouchPanel.ReadGesture();
				switch (gestureSample.GestureType)
				{
					case GestureType.FreeDrag:
						rotationX += gestureSample.Delta.X;
						rotationY -= gestureSample.Delta.Y;
						break;

					case GestureType.Pinch:
						const float fieldOfViewMin = 80;
						const float fieldOfViewMax = 20;
						const float gestureLengthToZoomScale = 10;

						Vector2 gestureDiff = gestureSample.Position - gestureSample.Position2;
						float gestureValue = gestureDiff.Length() / gestureLengthToZoomScale;

						if (previousGestureValue != null) /* Skip the first pinch event. */
						{
							cameraFieldOfView -= gestureValue - previousGestureValue.Value;
						}

						cameraFieldOfView = MathHelper.Clamp(
							cameraFieldOfView, fieldOfViewMax, fieldOfViewMin);

						previousGestureValue = gestureValue;
						break;

					case GestureType.PinchComplete:
						previousGestureValue = null;
						break;

					default:
						break;
				}
			}
		}
	}
}
